@@ -28,31 +28,11 @@
  * video codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
  * Foundation</ulink>, based on the VP3 codec.
  * </para>
- * <para>
- * The theora codec internally only supports encoding of images that are a
- * multiple of 16 pixels in both X and Y direction. It is however perfectly
- * possible to encode images with other dimensions because an arbitrary
- * rectangular cropping region can be set up. This element will automatically
- * set up a correct cropping region if the dimensions are not multiples of 16
- * pixels. The "border" and "center" properties control how this cropping
- * region will be set up.
- * </para>
- * <para>
- * To control the quality of the encoding, the "bitrate" and "quality"
- * properties can be used. These two properties are mutualy exclusive. Setting
- * the bitrate property will produce a constant bitrate (CBR) stream while
- * setting the quality property will produce a variable bitrate (VBR) stream.
- * </para>
  * <title>Example pipeline</title>
  * <programlisting>
  * gst-launch -v videotestsrc num-buffers=1000 ! theoraenc ! oggmux ! filesink location=videotestsrc.ogg
  * </programlisting>
- * This example pipeline will encode a test video source to theora muxed in an
- * ogg container. Refer to the theoradec documentation to decode the create
- * stream.
  * </refsect2>
- *
- * Last reviewed on 2006-03-01 (0.10.4)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,21 +40,18 @@
 #endif
 
 #include "gsttheoraenc.h"
-
 #include <string.h>
-#include <stdlib.h>             /* free */
-
 #include <gst/tag/tag.h>
 
+GST_DEBUG_CATEGORY (theoraenc_debug);
 #define GST_CAT_DEFAULT theoraenc_debug
-GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
 #define GST_TYPE_BORDER_MODE (gst_border_mode_get_type())
 static GType
 gst_border_mode_get_type (void)
 {
   static GType border_mode_type = 0;
-  static const GEnumValue border_mode[] = {
+  static GEnumValue border_mode[] = {
     {BORDER_NONE, "No Border", "none"},
     {BORDER_BLACK, "Black Border", "black"},
     {BORDER_MIRROR, "Mirror image in borders", "mirror"},
@@ -88,18 +65,9 @@ gst_border_mode_get_type (void)
   return border_mode_type;
 }
 
-/* taken from theora/lib/toplevel.c */
-static int
-_ilog (unsigned int v)
-{
-  int ret = 0;
-
-  while (v) {
-    ret++;
-    v >>= 1;
-  }
-  return (ret);
-}
+#define ROUND_UP_2(x) (((x) + 1) & ~1)
+#define ROUND_UP_4(x) (((x) + 3) & ~3)
+#define ROUND_UP_8(x) (((x) + 7) & ~7)
 
 #define THEORA_DEF_CENTER               TRUE
 #define THEORA_DEF_BORDER               BORDER_BLACK
@@ -113,6 +81,7 @@ _ilog (unsigned int v)
 #define THEORA_DEF_KEYFRAME_MINDISTANCE 8
 #define THEORA_DEF_NOISE_SENSITIVITY    1
 #define THEORA_DEF_SHARPNESS            0
+
 enum
 {
   ARG_0,
@@ -131,30 +100,12 @@ enum
   /* FILL ME */
 };
 
-/* this function does a straight granulepos -> timestamp conversion */
-static GstClockTime
-granulepos_to_timestamp (GstTheoraEnc * theoraenc, ogg_int64_t granulepos)
-{
-  guint64 iframe, pframe;
-  int shift = theoraenc->granule_shift;
-
-  if (granulepos < 0)
-    return GST_CLOCK_TIME_NONE;
-
-  iframe = granulepos >> shift;
-  pframe = granulepos - (iframe << shift);
-
-  /* num and den are 32 bit, so we can safely multiply with GST_SECOND */
-  return gst_util_uint64_scale ((guint64) (iframe + pframe),
-      GST_SECOND * theoraenc->info.fps_denominator,
-      theoraenc->info.fps_numerator);
-}
-
-static const GstElementDetails theora_enc_details =
-GST_ELEMENT_DETAILS ("Theora video encoder",
-    "Codec/Encoder/Video",
-    "encode raw YUV video to a theora stream",
-    "Wim Taymans <wim@fluendo.com>");
+static GstElementDetails theora_enc_details = {
+  "TheoraEnc",
+  "Codec/Encoder/Video",
+  "encode raw YUV video to a theora stream",
+  "Wim Taymans <wim@fluendo.com>",
+};
 
 static GstStaticPadTemplate theora_enc_sink_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -184,7 +135,6 @@ static void theora_enc_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void theora_enc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
-static void theora_enc_finalize (GObject * object);
 
 static void
 gst_theora_enc_base_init (gpointer g_class)
@@ -206,7 +156,6 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
 
   gobject_class->set_property = theora_enc_set_property;
   gobject_class->get_property = theora_enc_get_property;
-  gobject_class->finalize = theora_enc_finalize;
 
   g_object_class_install_property (gobject_class, ARG_CENTER,
       g_param_spec_boolean ("center", "Center",
@@ -287,24 +236,6 @@ gst_theora_enc_init (GstTheoraEnc * enc, GstTheoraEncClass * g_class)
   enc->keyframe_mindistance = THEORA_DEF_KEYFRAME_MINDISTANCE;
   enc->noise_sensitivity = THEORA_DEF_NOISE_SENSITIVITY;
   enc->sharpness = THEORA_DEF_SHARPNESS;
-
-  enc->granule_shift = _ilog (enc->info.keyframe_frequency_force - 1);
-  GST_DEBUG_OBJECT (enc,
-      "keyframe_frequency_force is %d, granule shift is %d",
-      enc->info.keyframe_frequency_force, enc->granule_shift);
-}
-
-static void
-theora_enc_finalize (GObject * object)
-{
-  GstTheoraEnc *enc = GST_THEORA_ENC (object);
-
-  GST_DEBUG_OBJECT (enc, "Finalizing");
-  theora_clear (&enc->state);
-  theora_comment_clear (&enc->comment);
-  theora_info_clear (&enc->info);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gboolean
@@ -320,7 +251,6 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   gst_structure_get_fraction (structure, "framerate", &fps_n, &fps_d);
   par = gst_structure_get_value (structure, "pixel-aspect-ratio");
 
-  theora_info_clear (&enc->info);
   theora_info_init (&enc->info);
   /* Theora has a divisible-by-sixteen restriction for the encoded video size but
    * we can define a visible area using the frame_width/frame_height */
@@ -332,8 +262,8 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   /* center image if needed */
   if (enc->center) {
     /* make sure offset is even, for easier decoding */
-    enc->offset_x = GST_ROUND_UP_2 ((enc->info_width - enc->width) / 2);
-    enc->offset_y = GST_ROUND_UP_2 ((enc->info_height - enc->height) / 2);
+    enc->offset_x = ROUND_UP_2 ((enc->info_width - enc->width) / 2);
+    enc->offset_y = ROUND_UP_2 ((enc->info_height - enc->height) / 2);
   } else {
     enc->offset_x = 0;
     enc->offset_y = 0;
@@ -341,8 +271,8 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   enc->info.offset_x = enc->offset_x;
   enc->info.offset_y = enc->offset_y;
 
-  enc->info.fps_numerator = enc->fps_n = fps_n;
-  enc->info.fps_denominator = enc->fps_d = fps_d;
+  enc->info.fps_numerator = fps_n;
+  enc->info.fps_denominator = fps_d;
   if (par) {
     enc->info.aspect_numerator = gst_value_get_fraction_numerator (par);
     enc->info.aspect_denominator = gst_value_get_fraction_denominator (par);
@@ -368,31 +298,11 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
   enc->info.noise_sensitivity = enc->noise_sensitivity;
   enc->info.sharpness = enc->sharpness;
 
-  /* as done in theora */
-  enc->granule_shift = _ilog (enc->info.keyframe_frequency_force - 1);
-  GST_DEBUG_OBJECT (enc,
-      "keyframe_frequency_force is %d, granule shift is %d",
-      enc->info.keyframe_frequency_force, enc->granule_shift);
-
   theora_encode_init (&enc->state, &enc->info);
 
-  gst_object_unref (enc);
-
   return TRUE;
 }
 
-static guint64
-granulepos_add (guint64 granulepos, guint64 addend, gint shift)
-{
-  guint64 iframe, pframe;
-
-  iframe = granulepos >> shift;
-  pframe = granulepos - (iframe << shift);
-  iframe += addend;
-
-  return (iframe << shift) + pframe;
-}
-
 /* prepare a buffer for transmission by passing data through libtheora */
 static GstFlowReturn
 theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet,
@@ -407,15 +317,9 @@ theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet,
     goto no_buffer;
 
   memcpy (GST_BUFFER_DATA (buf), packet->packet, packet->bytes);
-  /* see ext/ogg/README; OFFSET_END takes "our" granulepos, OFFSET its
-   * time representation */
-  GST_BUFFER_OFFSET_END (buf) =
-      granulepos_add (packet->granulepos, enc->granulepos_offset,
-      enc->granule_shift);
-  GST_BUFFER_OFFSET (buf) = granulepos_to_timestamp (enc,
-      GST_BUFFER_OFFSET_END (buf));
-
-  GST_BUFFER_TIMESTAMP (buf) = timestamp + enc->timestamp_offset;
+  GST_BUFFER_OFFSET (buf) = enc->bytes_out;
+  GST_BUFFER_OFFSET_END (buf) = packet->granulepos;
+  GST_BUFFER_TIMESTAMP (buf) = timestamp;
   GST_BUFFER_DURATION (buf) = duration;
 
   /* the second most significant bit of the first data byte is cleared
@@ -480,38 +384,23 @@ theora_set_header_on_caps (GstCaps * caps, GstBuffer * buf1,
   GST_BUFFER_FLAG_SET (buf2, GST_BUFFER_FLAG_IN_CAPS);
   GST_BUFFER_FLAG_SET (buf3, GST_BUFFER_FLAG_IN_CAPS);
 
-  /* Copy buffers, because we can't use the originals -
-   * it creates a circular refcount with the caps<->buffers */
-  buf1 = gst_buffer_copy (buf1);
-  buf2 = gst_buffer_copy (buf2);
-  buf3 = gst_buffer_copy (buf3);
-
-  /* put copies of the buffers in a fixed list */
+  /* put buffers in a fixed list */
   g_value_init (&array, GST_TYPE_ARRAY);
-
   g_value_init (&value, GST_TYPE_BUFFER);
   gst_value_set_buffer (&value, buf1);
   gst_value_array_append_value (&array, &value);
   g_value_unset (&value);
-
   g_value_init (&value, GST_TYPE_BUFFER);
   gst_value_set_buffer (&value, buf2);
   gst_value_array_append_value (&array, &value);
   g_value_unset (&value);
-
   g_value_init (&value, GST_TYPE_BUFFER);
   gst_value_set_buffer (&value, buf3);
   gst_value_array_append_value (&array, &value);
-  g_value_unset (&value);
-
   gst_structure_set_value (structure, "streamheader", &array);
+  g_value_unset (&value);
   g_value_unset (&array);
 
-  /* Unref our copies */
-  gst_buffer_unref (buf1);
-  gst_buffer_unref (buf2);
-  gst_buffer_unref (buf3);
-
   return caps;
 }
 
@@ -528,12 +417,10 @@ theora_enc_sink_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_EOS:
       /* push last packet with eos flag */
       while (theora_encode_packetout (&enc->state, 1, &op)) {
-        /* See comment in the chain function */
-        GstClockTime next_time = theora_granule_time (&enc->state,
-            granulepos_add (op.granulepos, 1, enc->granule_shift)) * GST_SECOND;
+        GstClockTime out_time =
+            theora_granule_time (&enc->state, op.granulepos) * GST_SECOND;
 
-        theora_push_packet (enc, &op, enc->next_ts, next_time - enc->next_ts);
-        enc->next_ts = next_time;
+        theora_push_packet (enc, &op, out_time, GST_SECOND / enc->fps);
       }
       res = gst_pad_push_event (enc->srcpad, event);
       break;
@@ -560,9 +447,6 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     GstCaps *caps;
     GstBuffer *buf1, *buf2, *buf3;
 
-    enc->granulepos_offset = 0;
-    enc->timestamp_offset = 0;
-
     /* Theora streams begin with three headers; the initial header (with
        most of the codec setup parameters) which is mandated by the Ogg
        bitstream spec.  The second header holds any comment fields.  The
@@ -572,34 +456,25 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
 
     /* first packet will get its own page automatically */
     theora_encode_header (&enc->state, &op);
-    ret = theora_buffer_from_packet (enc, &op, GST_CLOCK_TIME_NONE,
-        GST_CLOCK_TIME_NONE, &buf1);
+    ret = theora_buffer_from_packet (enc, &op, 0, 0, &buf1);
     if (ret != GST_FLOW_OK) {
       goto header_buffer_alloc;
     }
 
     /* create the remaining theora headers */
-    theora_comment_clear (&enc->comment);
     theora_comment_init (&enc->comment);
-
+    /* Currently leaks due to libtheora API brokenness, I don't think we can
+     * portably work around it. Leaks ~50 bytes per encoder instance, so not a
+     * huge problem. */
     theora_encode_comment (&enc->comment, &op);
-    ret = theora_buffer_from_packet (enc, &op, GST_CLOCK_TIME_NONE,
-        GST_CLOCK_TIME_NONE, &buf2);
-    /* Theora expects us to put this packet buffer into an ogg page,
-     * in which case it becomes the ogg library's responsibility to
-     * free it. Since we're copying and outputting a gst_buffer,
-     * we need to free it ourselves. */
-    if (op.packet)
-      free (op.packet);
-
+    ret = theora_buffer_from_packet (enc, &op, 0, 0, &buf2);
     if (ret != GST_FLOW_OK) {
       gst_buffer_unref (buf1);
       goto header_buffer_alloc;
     }
 
     theora_encode_tables (&enc->state, &op);
-    ret = theora_buffer_from_packet (enc, &op, GST_CLOCK_TIME_NONE,
-        GST_CLOCK_TIME_NONE, &buf3);
+    ret = theora_buffer_from_packet (enc, &op, 0, 0, &buf3);
     if (ret != GST_FLOW_OK) {
       gst_buffer_unref (buf1);
       gst_buffer_unref (buf2);
@@ -616,8 +491,6 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     gst_buffer_set_caps (buf2, caps);
     gst_buffer_set_caps (buf3, caps);
 
-    gst_caps_unref (caps);
-
     /* push out the header buffers */
     if ((ret = theora_push_buffer (enc, buf1)) != GST_FLOW_OK) {
       gst_buffer_unref (buf2);
@@ -631,12 +504,6 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
     if ((ret = theora_push_buffer (enc, buf3)) != GST_FLOW_OK) {
       goto header_push;
     }
-
-    enc->granulepos_offset =
-        gst_util_uint64_scale (GST_BUFFER_TIMESTAMP (buffer), enc->fps_n,
-        GST_SECOND * enc->fps_d);
-    enc->timestamp_offset = GST_BUFFER_TIMESTAMP (buffer);
-    enc->next_ts = 0;
   }
 
   {
@@ -682,8 +549,8 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
       cheight = height / 2;
 
       /* source strides as defined in videotestsrc */
-      src_y_stride = GST_ROUND_UP_4 (width);
-      src_uv_stride = GST_ROUND_UP_8 (width) / 2;
+      src_y_stride = ROUND_UP_4 (width);
+      src_uv_stride = ROUND_UP_8 (width) / 2;
 
       /* destination strides from the real picture width */
       dst_y_stride = enc->info_width;
@@ -700,8 +567,8 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
       dest_v = yuv.v = yuv.u + y_size / 4;
 
       src_y = GST_BUFFER_DATA (buffer);
-      src_u = src_y + src_y_stride * GST_ROUND_UP_2 (height);
-      src_v = src_u + src_uv_stride * GST_ROUND_UP_2 (height) / 2;
+      src_u = src_y + src_y_stride * ROUND_UP_2 (height);
+      src_v = src_u + src_uv_stride * ROUND_UP_2 (height) / 2;
 
       if (enc->border != BORDER_NONE) {
         /* fill top border */
@@ -788,18 +655,11 @@ theora_enc_chain (GstPad * pad, GstBuffer * buffer)
 
     ret = GST_FLOW_OK;
     while (theora_encode_packetout (&enc->state, 0, &op)) {
-      /* This is where we hack around theora's broken idea of what granulepos
-         is -- normally we wouldn't need to add the 1, because granulepos
-         should be the presentation time of the last sample in the packet, but
-         theora starts with 0 instead of 1... */
-      GstClockTime next_time;
+      GstClockTime out_time;
 
-      next_time = theora_granule_time (&enc->state,
-          granulepos_add (op.granulepos, 1, enc->granule_shift)) * GST_SECOND;
-      ret =
-          theora_push_packet (enc, &op, enc->next_ts, next_time - enc->next_ts);
-      enc->next_ts = next_time;
-      if (ret != GST_FLOW_OK)
+      out_time = theora_granule_time (&enc->state, op.granulepos) * GST_SECOND;
+      if ((ret = theora_push_packet (enc, &op, out_time, GST_SECOND / enc->fps))
+          != GST_FLOW_OK)
         goto data_push;
     }
     gst_buffer_unref (buffer);
@@ -842,10 +702,10 @@ theora_enc_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      GST_DEBUG_OBJECT (enc, "READY->PAUSED Initing theora state");
       theora_info_init (&enc->info);
       theora_comment_init (&enc->comment);
       enc->packetno = 0;
+      enc->initial_delay = 0;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -859,7 +719,6 @@ theora_enc_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      GST_DEBUG_OBJECT (enc, "PAUSED->READY Clearing theora state");
       theora_clear (&enc->state);
       theora_comment_clear (&enc->comment);
       theora_info_clear (&enc->info);
