@@ -18,44 +18,38 @@
  */
 
 
-#ifndef __GST_VORBIS_ENC_H__
-#define __GST_VORBIS_ENC_H__
+#ifndef __VORBISENC_H__
+#define __VORBISENC_H__
 
 
 #include <gst/gst.h>
 
 #include <vorbis/codec.h>
 
-G_BEGIN_DECLS
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
 
 #define GST_TYPE_VORBISENC \
-  (gst_vorbis_enc_get_type())
+  (vorbisenc_get_type())
 #define GST_VORBISENC(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VORBISENC,GstVorbisEnc))
 #define GST_VORBISENC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VORBISENC,GstVorbisEncClass))
 #define GST_IS_VORBISENC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VORBISENC))
-#define GST_IS_VORBISENC_CLASS(klass) \
+#define GST_IS_VORBISENC_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VORBISENC))
 
 typedef struct _GstVorbisEnc GstVorbisEnc;
 typedef struct _GstVorbisEncClass GstVorbisEncClass;
 
-/**
- * GstVorbisEnc:
- *
- * Opaque data structure.
- */
 struct _GstVorbisEnc {
   GstElement       element;
 
   GstPad          *sinkpad;
   GstPad          *srcpad;
 
-  GstCaps         *srccaps;
-  GstCaps         *sinkcaps;
-
   vorbis_info      vi; /* struct that stores all the static vorbis bitstream
                                                             settings */
   vorbis_comment   vc; /* struct that stores all the user comments */
@@ -75,11 +69,7 @@ struct _GstVorbisEnc {
 
   guint64          samples_in;
   guint64          bytes_out;
-  GstClockTime     next_ts;
-  GstClockTime     expected_ts;
-  gboolean         next_discont;
-  guint64          granulepos_offset;
-  gint64           subgranule_offset;
+  GstClockTime     prev_ts;
 
   GstTagList *     tags;
 
@@ -92,8 +82,12 @@ struct _GstVorbisEncClass {
   GstElementClass parent_class;
 };
 
-GType gst_vorbis_enc_get_type(void);
+GType vorbisenc_get_type(void);
 
-G_END_DECLS
 
-#endif /* __GST_VORBIS_ENC_H__ */
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
+
+#endif /* __VORBISENC_H__ */
