@@ -50,39 +50,14 @@ G_BEGIN_DECLS
  * Thomas, 6 September 2002 */
 
 /* conversion macros */
-/**
- * GST_FRAMES_TO_CLOCK_TIME:
- * @frames: sample frames
- * @rate: sampling rate
- * 
- * Calculate clocktime from sample @frames and @rate.
- */
 #define GST_FRAMES_TO_CLOCK_TIME(frames, rate) \
-  ((GstClockTime) gst_util_uint64_scale_round (frames, GST_SECOND, rate))
+  ((GstClockTime) (((gdouble) frames / rate) * GST_SECOND))
 
-/**
- * GST_CLOCK_TIME_TO_FRAMES:
- * @clocktime: clock time
- * @rate: sampling rate
- * 
- * Calculate frames from @clocktime and sample @rate.
- */
 #define GST_CLOCK_TIME_TO_FRAMES(clocktime, rate) \
-  gst_util_uint64_scale_round (clocktime, rate, GST_SECOND)
+  ((gint64) (((gdouble) clocktime / GST_SECOND) * rate))
 
-/**
- * GST_AUDIO_DEF_RATE:
- * 
- * Standard sampling rate used in consumer audio.
- */
 #define GST_AUDIO_DEF_RATE 44100
 
-/**
- * GST_AUDIO_INT_PAD_TEMPLATE_CAPS:
- * 
- * Template caps for integer audio. Can be used when defining a 
- * #GstStaticPadTemplate
- */
 #define GST_AUDIO_INT_PAD_TEMPLATE_CAPS \
   "audio/x-raw-int, " \
   "rate = (int) [ 1, MAX ], " \
@@ -92,12 +67,8 @@ G_BEGIN_DECLS
   "depth = (int) [ 1, 32 ], " \
   "signed = (boolean) { true, false }"
 
-/**
- * GST_AUDIO_INT_STANDARD_PAD_TEMPLATE_CAPS:
- * 
- * Template caps for 16bit integer stereo audio in native byte-order.
- * Can be used when defining a #GstStaticPadTemplate
- */
+
+/* "standard" int audio is native order, 16 bit stereo. */
 #define GST_AUDIO_INT_STANDARD_PAD_TEMPLATE_CAPS \
   "audio/x-raw-int, " \
   "rate = (int) [ 1, MAX ], " \
@@ -107,12 +78,6 @@ G_BEGIN_DECLS
   "depth = (int) 16, " \
   "signed = (boolean) true"
 
-/**
- * GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS:
- * 
- * Template caps for float audio. Can be used when defining a 
- * #GstStaticPadTemplate
- */
 #define GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS \
   "audio/x-raw-float, " \
   "rate = (int) [ 1, MAX ], " \
@@ -120,12 +85,7 @@ G_BEGIN_DECLS
   "endianness = (int) { LITTLE_ENDIAN , BIG_ENDIAN }, " \
   "width = (int) { 32, 64 }"
 
-/**
- * GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS:
- * 
- * Template caps for 32bit float mono audio in native byte-order.
- * Can be used when defining a #GstStaticPadTemplate
- */
+/* "standard" float audio is native order, 32 bit mono. */
 #define GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS \
   "audio/x-raw-float, " \
   "width = (int) 32, " \
@@ -150,29 +110,16 @@ GstClockTime gst_audio_duration_from_pad_buffer (GstPad * pad, GstBuffer * buf);
 gboolean gst_audio_is_buffer_framed     (GstPad* pad, GstBuffer* buf);
 
 /* functions useful for _getcaps functions */
-/**
- * GstAudioFieldFlag:
- *
- * Do not use anymore.
- *
- * Deprecated: use gst_structure_set() directly
- */
-#ifndef GST_DISABLE_DEPRECATED
 typedef enum {
   GST_AUDIO_FIELD_RATE          = (1 << 0),
   GST_AUDIO_FIELD_CHANNELS      = (1 << 1),
   GST_AUDIO_FIELD_ENDIANNESS    = (1 << 2),
   GST_AUDIO_FIELD_WIDTH         = (1 << 3),
   GST_AUDIO_FIELD_DEPTH         = (1 << 4),
-  GST_AUDIO_FIELD_SIGNED        = (1 << 5)
+  GST_AUDIO_FIELD_SIGNED        = (1 << 5),
 } GstAudioFieldFlag;
-#endif
 
-#ifndef GST_DISABLE_DEPRECATED
 void gst_audio_structure_set_int (GstStructure *structure, GstAudioFieldFlag flag);
-#endif /* GST_DISABLE_DEPRECATED */
-
-GstBuffer *gst_audio_buffer_clip (GstBuffer *buffer, GstSegment *segment, gint rate, gint frame_size);
 
 G_END_DECLS
 
