@@ -35,8 +35,6 @@ GST_DEBUG_CATEGORY_STATIC (gst_audio_sink_debug);
         (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_AUDIORING_BUFFER,GstAudioRingBufferClass))
 #define GST_AUDIORING_BUFFER_GET_CLASS(obj) \
         (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_AUDIORING_BUFFER, GstAudioRingBufferClass))
-#define GST_AUDIORING_BUFFER_CAST(obj)        \
-        ((GstAudioRingBuffer *)obj)
 #define GST_IS_AUDIORING_BUFFER(obj)     \
         (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_AUDIORING_BUFFER))
 #define GST_IS_AUDIORING_BUFFER_CLASS(obj)\
@@ -79,7 +77,6 @@ static gboolean gst_audioringbuffer_acquire (GstRingBuffer * buf,
     GstRingBufferSpec * spec);
 static gboolean gst_audioringbuffer_release (GstRingBuffer * buf);
 static gboolean gst_audioringbuffer_start (GstRingBuffer * buf);
-static gboolean gst_audioringbuffer_pause (GstRingBuffer * buf);
 static gboolean gst_audioringbuffer_stop (GstRingBuffer * buf);
 static guint gst_audioringbuffer_delay (GstRingBuffer * buf);
 
@@ -121,7 +118,7 @@ gst_audioringbuffer_class_init (GstAudioRingBufferClass * klass)
   gstobject_class = (GstObjectClass *) klass;
   gstringbuffer_class = (GstRingBufferClass *) klass;
 
-  ring_parent_class = g_type_class_peek_parent (klass);
+  ring_parent_class = g_type_class_ref (GST_TYPE_RING_BUFFER);
 
   gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_audioringbuffer_dispose);
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_audioringbuffer_finalize);
@@ -135,7 +132,7 @@ gst_audioringbuffer_class_init (GstAudioRingBufferClass * klass)
   gstringbuffer_class->release =
       GST_DEBUG_FUNCPTR (gst_audioringbuffer_release);
   gstringbuffer_class->start = GST_DEBUG_FUNCPTR (gst_audioringbuffer_start);
-  gstringbuffer_class->pause = GST_DEBUG_FUNCPTR (gst_audioringbuffer_pause);
+  gstringbuffer_class->pause = GST_DEBUG_FUNCPTR (gst_audioringbuffer_stop);
   gstringbuffer_class->resume = GST_DEBUG_FUNCPTR (gst_audioringbuffer_start);
   gstringbuffer_class->stop = GST_DEBUG_FUNCPTR (gst_audioringbuffer_stop);
 
@@ -154,7 +151,7 @@ audioringbuffer_thread_func (GstRingBuffer * buf)
 {
   GstAudioSink *sink;
   GstAudioSinkClass *csink;
-  GstAudioRingBuffer *abuf = GST_AUDIORING_BUFFER_CAST (buf);
+  GstAudioRingBuffer *abuf = GST_AUDIORING_BUFFER (buf);
   WriteFunc writefunc;
 
   sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
@@ -244,10 +241,6 @@ gst_audioringbuffer_dispose (GObject * object)
 static void
 gst_audioringbuffer_finalize (GObject * object)
 {
-  GstAudioRingBuffer *ringbuffer = GST_AUDIORING_BUFFER_CAST (object);
-
-  g_cond_free (ringbuffer->cond);
-
   G_OBJECT_CLASS (ring_parent_class)->finalize (object);
 }
 
@@ -324,7 +317,7 @@ gst_audioringbuffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
   buf->data = gst_buffer_new_and_alloc (spec->segtotal * spec->segsize);
   memset (GST_BUFFER_DATA (buf->data), 0, GST_BUFFER_SIZE (buf->data));
 
-  abuf = GST_AUDIORING_BUFFER_CAST (buf);
+  abuf = GST_AUDIORING_BUFFER (buf);
   abuf->running = TRUE;
 
   sink->thread =
@@ -352,10 +345,9 @@ gst_audioringbuffer_release (GstRingBuffer * buf)
 
   sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
   csink = GST_AUDIO_SINK_GET_CLASS (sink);
-  abuf = GST_AUDIORING_BUFFER_CAST (buf);
+  abuf = GST_AUDIORING_BUFFER (buf);
 
   abuf->running = FALSE;
-  GST_DEBUG ("signal wait");
   GST_AUDIORING_BUFFER_SIGNAL (buf);
   GST_OBJECT_UNLOCK (buf);
 
@@ -396,25 +388,6 @@ gst_audioringbuffer_start (GstRingBuffer * buf)
   return TRUE;
 }
 
-static gboolean
-gst_audioringbuffer_pause (GstRingBuffer * buf)
-{
-  GstAudioSink *sink;
-  GstAudioSinkClass *csink;
-
-  sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
-  csink = GST_AUDIO_SINK_GET_CLASS (sink);
-
-  /* unblock any pending writes to the audio device */
-  if (csink->reset) {
-    GST_DEBUG ("reset...");
-    csink->reset (sink);
-    GST_DEBUG ("reset done");
-  }
-
-  return TRUE;
-}
-
 static gboolean
 gst_audioringbuffer_stop (GstRingBuffer * buf)
 {
@@ -424,7 +397,7 @@ gst_audioringbuffer_stop (GstRingBuffer * buf)
 
   sink = GST_AUDIO_SINK (GST_OBJECT_PARENT (buf));
   csink = GST_AUDIO_SINK_GET_CLASS (sink);
-  abuf = GST_AUDIORING_BUFFER_CAST (buf);
+  abuf = GST_AUDIORING_BUFFER (buf);
 
   /* unblock any pending writes to the audio device */
   if (csink->reset) {
