@@ -34,9 +34,8 @@ enum
   LAST_SIGNAL
 };
 
-#define DEFAULT_BUFFER_TIME     ((200 * GST_MSECOND) / GST_USECOND)
-#define DEFAULT_LATENCY_TIME    ((10 * GST_MSECOND) / GST_USECOND)
-
+#define DEFAULT_BUFFER_TIME     500 * GST_USECOND
+#define DEFAULT_LATENCY_TIME    10 * GST_USECOND
 enum
 {
   PROP_0,
@@ -61,14 +60,11 @@ static GstStateChangeReturn gst_base_audio_src_change_state (GstElement *
     element, GstStateChange transition);
 
 static GstClock *gst_base_audio_src_provide_clock (GstElement * elem);
-static gboolean gst_base_audio_src_set_clock (GstElement * elem,
-    GstClock * clock);
 static GstClockTime gst_base_audio_src_get_time (GstClock * clock,
     GstBaseAudioSrc * src);
 
-static GstFlowReturn gst_base_audio_src_create (GstBaseSrc * bsrc,
-    guint64 offset, guint length, GstBuffer ** buf);
-static gboolean gst_base_audio_src_check_get_range (GstBaseSrc * bsrc);
+static GstFlowReturn gst_base_audio_src_create (GstPushSrc * psrc,
+    GstBuffer ** buf);
 
 static gboolean gst_base_audio_src_event (GstBaseSrc * bsrc, GstEvent * event);
 static void gst_base_audio_src_get_times (GstBaseSrc * bsrc,
@@ -85,6 +81,7 @@ gst_base_audio_src_base_init (gpointer g_class)
 static void
 gst_base_audio_src_class_init (GstBaseAudioSrcClass * klass)
 {
+  gchar *longdesc;
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
@@ -100,30 +97,33 @@ gst_base_audio_src_class_init (GstBaseAudioSrcClass * klass)
   gobject_class->get_property =
       GST_DEBUG_FUNCPTR (gst_base_audio_src_get_property);
 
-  g_object_class_install_property (gobject_class, PROP_BUFFER_TIME,
-      g_param_spec_int64 ("buffer-time", "Buffer Time",
-          "Size of audio buffer in microseconds", 1,
+  longdesc =
+      g_strdup_printf
+      ("Size of audio buffer in microseconds (use -1 for default of %"
+      G_GUINT64_FORMAT " us)", DEFAULT_BUFFER_TIME / GST_USECOND);
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BUFFER_TIME,
+      g_param_spec_int64 ("buffer-time", "Buffer Time", longdesc, -1,
           G_MAXINT64, DEFAULT_BUFFER_TIME, G_PARAM_READWRITE));
-
-  g_object_class_install_property (gobject_class, PROP_LATENCY_TIME,
-      g_param_spec_int64 ("latency-time", "Latency Time",
-          "Audio latency in microseconds", 1,
+  g_free (longdesc);
+  longdesc =
+      g_strdup_printf ("Audio latency in microseconds (use -1 for default of %"
+      G_GUINT64_FORMAT " us)", DEFAULT_LATENCY_TIME / GST_USECOND);
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_LATENCY_TIME,
+      g_param_spec_int64 ("latency-time", "Latency Time", longdesc, -1,
           G_MAXINT64, DEFAULT_LATENCY_TIME, G_PARAM_READWRITE));
+  g_free (longdesc);
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_base_audio_src_change_state);
   gstelement_class->provide_clock =
       GST_DEBUG_FUNCPTR (gst_base_audio_src_provide_clock);
-  gstelement_class->set_clock =
-      GST_DEBUG_FUNCPTR (gst_base_audio_src_set_clock);
 
   gstbasesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_base_audio_src_setcaps);
   gstbasesrc_class->event = GST_DEBUG_FUNCPTR (gst_base_audio_src_event);
   gstbasesrc_class->get_times =
       GST_DEBUG_FUNCPTR (gst_base_audio_src_get_times);
-  gstbasesrc_class->create = GST_DEBUG_FUNCPTR (gst_base_audio_src_create);
-  gstbasesrc_class->check_get_range =
-      GST_DEBUG_FUNCPTR (gst_base_audio_src_check_get_range);
+
+  gstpushsrc_class->create = GST_DEBUG_FUNCPTR (gst_base_audio_src_create);
 }
 
 static void
@@ -132,70 +132,22 @@ gst_base_audio_src_init (GstBaseAudioSrc * baseaudiosrc,
 {
   baseaudiosrc->buffer_time = DEFAULT_BUFFER_TIME;
   baseaudiosrc->latency_time = DEFAULT_LATENCY_TIME;
-  /* reset blocksize we use latency time to calculate a more useful 
-   * value based on negotiated format. */
-  GST_BASE_SRC (baseaudiosrc)->blocksize = 0;
 
   baseaudiosrc->clock = gst_audio_clock_new ("clock",
       (GstAudioClockGetTimeFunc) gst_base_audio_src_get_time, baseaudiosrc);
 
   gst_pad_set_fixatecaps_function (GST_BASE_SRC_PAD (baseaudiosrc),
       gst_base_audio_src_fixate);
-
-  /* we are always a live source */
-  gst_base_src_set_live (GST_BASE_SRC (baseaudiosrc), TRUE);
-  gst_base_src_set_format (GST_BASE_SRC (baseaudiosrc), GST_FORMAT_TIME);
-}
-
-static gboolean
-gst_base_audio_src_set_clock (GstElement * elem, GstClock * clock)
-{
-  GstBaseAudioSrc *src;
-
-  src = GST_BASE_AUDIO_SRC (elem);
-
-  /* FIXME, we cannot slave to another clock yet, better fail 
-   * than to give a bad user experience (tm). */
-  if (clock && clock != src->clock)
-    goto wrong_clock;
-
-  return TRUE;
-
-  /* ERRORS */
-wrong_clock:
-  {
-    /* no error message, this method is called with the parent
-     * lock helt.. sigh.. long live recursive locks.. */
-    GST_DEBUG_OBJECT (src, "Cannot operate with this clock.");
-    return FALSE;
-  }
 }
 
 static GstClock *
 gst_base_audio_src_provide_clock (GstElement * elem)
 {
   GstBaseAudioSrc *src;
-  GstClock *clock;
 
   src = GST_BASE_AUDIO_SRC (elem);
 
-  /* we have no ringbuffer (must be NULL state) */
-  if (src->ringbuffer == NULL)
-    goto wrong_state;
-
-  if (!gst_ring_buffer_is_acquired (src->ringbuffer))
-    goto wrong_state;
-
-  clock = GST_CLOCK_CAST (gst_object_ref (src->clock));
-
-  return clock;
-
-  /* ERRORS */
-wrong_state:
-  {
-    GST_DEBUG_OBJECT (src, "ringbuffer not acquired");
-    return NULL;
-  }
+  return GST_CLOCK (gst_object_ref (GST_OBJECT (src->clock)));
 }
 
 static GstClockTime
@@ -204,27 +156,16 @@ gst_base_audio_src_get_time (GstClock * clock, GstBaseAudioSrc * src)
   guint64 samples;
   GstClockTime result;
 
-  if (G_UNLIKELY (src->ringbuffer == NULL || src->ringbuffer->spec.rate == 0))
-    return GST_CLOCK_TIME_NONE;
+  if (src->ringbuffer == NULL || src->ringbuffer->spec.rate == 0)
+    return 0;
 
   samples = gst_ring_buffer_samples_done (src->ringbuffer);
 
-  result = gst_util_uint64_scale_int (samples, GST_SECOND,
-      src->ringbuffer->spec.rate);
+  result = samples * GST_SECOND / src->ringbuffer->spec.rate;
 
   return result;
 }
 
-static gboolean
-gst_base_audio_src_check_get_range (GstBaseSrc * bsrc)
-{
-  /* we allow limited pull base operation of which the details
-   * will eventually exposed in an as of yet non-existing query.
-   * Basically pulling can be done on any number of bytes as long
-   * as the offset is -1 or sequentially increasing. */
-  return TRUE;
-}
-
 static void
 gst_base_audio_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -370,86 +311,52 @@ gst_base_audio_src_event (GstBaseSrc * bsrc, GstEvent * event)
 }
 
 static GstFlowReturn
-gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
-    GstBuffer ** outbuf)
+gst_base_audio_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
 {
-  GstBaseAudioSrc *src = GST_BASE_AUDIO_SRC (bsrc);
+  GstBaseAudioSrc *src = GST_BASE_AUDIO_SRC (psrc);
   GstBuffer *buf;
   guchar *data;
-  guint samples;
+  guint len, samples;
   guint res;
   guint64 sample;
-  gint bps;
-  GstRingBuffer *ringbuffer;
 
-  ringbuffer = src->ringbuffer;
-
-  if (G_UNLIKELY (!gst_ring_buffer_is_acquired (ringbuffer)))
+  if (!gst_ring_buffer_is_acquired (src->ringbuffer))
     goto wrong_state;
 
-  bps = ringbuffer->spec.bytes_per_sample;
+  buf = gst_buffer_new_and_alloc (src->ringbuffer->spec.segsize);
 
-  if ((length == 0 && bsrc->blocksize == 0) || length == -1)
-    /* no length given, use the default segment size */
-    length = ringbuffer->spec.segsize;
-  else
-    /* make sure we round down to an integral number of samples */
-    length -= length % bps;
+  data = GST_BUFFER_DATA (buf);
+  len = GST_BUFFER_SIZE (buf);
 
-  /* calculate the sequentially next sample we need to read */
-  sample = (src->next_sample != -1 ? src->next_sample : 0);
-
-  if (G_UNLIKELY (offset != -1)) {
-    /* if a specific offset was given it must be the next
-     * sequential offset we expect or we fail. */
-    if (offset / bps != sample)
-      goto wrong_offset;
+  if (src->next_sample != -1) {
+    sample = src->next_sample;
+  } else {
+    sample = 0;
   }
 
-  /* get the number of samples to read */
-  samples = length / bps;
+  samples = len / src->ringbuffer->spec.bytes_per_sample;
 
-  /* FIXME, using a bufferpool would be nice here */
-  buf = gst_buffer_new_and_alloc (length);
-  data = GST_BUFFER_DATA (buf);
-
-  res = gst_ring_buffer_read (ringbuffer, sample, data, samples);
-  if (G_UNLIKELY (res == -1))
+  res = gst_ring_buffer_read (src->ringbuffer, sample, data, samples);
+  if (res == -1)
     goto stopped;
 
-  /* FIXME, we timestamp against our own clock, also handle the case
-   * where we are slaved to another clock. We currently refuse to accept
-   * any other clock than the one we provide, so this code is fine for
-   * now. */
-  GST_BUFFER_TIMESTAMP (buf) = gst_util_uint64_scale_int (sample,
-      GST_SECOND, ringbuffer->spec.rate);
   src->next_sample = sample + samples;
-  GST_BUFFER_DURATION (buf) = gst_util_uint64_scale_int (src->next_sample,
-      GST_SECOND, ringbuffer->spec.rate) - GST_BUFFER_TIMESTAMP (buf);
 
-  gst_buffer_set_caps (buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (bsrc)));
+  gst_buffer_set_caps (buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (psrc)));
 
   *outbuf = buf;
 
   return GST_FLOW_OK;
 
-  /* ERRORS */
 wrong_state:
   {
-    GST_DEBUG_OBJECT (src, "ringbuffer in wrong state");
+    GST_DEBUG ("ringbuffer in wrong state");
     return GST_FLOW_WRONG_STATE;
   }
-wrong_offset:
-  {
-    GST_ELEMENT_ERROR (src, RESOURCE, SEEK,
-        (NULL), ("resource can only be operated on sequentially but offset %"
-            G_GUINT64_FORMAT " was given", offset));
-    return GST_FLOW_ERROR;
-  }
 stopped:
   {
     gst_buffer_unref (buf);
-    GST_DEBUG_OBJECT (src, "ringbuffer stopped");
+    GST_DEBUG ("ringbuffer stopped");
     return GST_FLOW_WRONG_STATE;
   }
 }
@@ -464,8 +371,9 @@ gst_base_audio_src_create_ringbuffer (GstBaseAudioSrc * src)
   if (bclass->create_ringbuffer)
     buffer = bclass->create_ringbuffer (src);
 
-  if (G_LIKELY (buffer))
-    gst_object_set_parent (GST_OBJECT_CAST (buffer), GST_OBJECT_CAST (src));
+  if (buffer) {
+    gst_object_set_parent (GST_OBJECT (buffer), GST_OBJECT (src));
+  }
 
   return buffer;
 }
@@ -499,7 +407,6 @@ gst_base_audio_src_change_state (GstElement * element,
       gst_ring_buffer_set_flushing (src->ringbuffer, FALSE);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      gst_ring_buffer_may_start (src->ringbuffer, TRUE);
       break;
     default:
       break;
@@ -509,7 +416,6 @@ gst_base_audio_src_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      gst_ring_buffer_may_start (src->ringbuffer, FALSE);
       gst_ring_buffer_pause (src->ringbuffer);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
@@ -519,7 +425,7 @@ gst_base_audio_src_change_state (GstElement * element,
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       gst_ring_buffer_close_device (src->ringbuffer);
-      gst_object_unparent (GST_OBJECT_CAST (src->ringbuffer));
+      gst_object_unref (src->ringbuffer);
       src->ringbuffer = NULL;
       break;
     default:
