@@ -19,25 +19,6 @@
  */
 /* Element-Checklist-Version: 5 */
 
-/**
- * SECTION:element-audioresample
- *
- * <refsect2>
- * Audioresample resamples raw audio buffers to different sample rates using
- * a configurable windowing function to enhance quality.
- * <title>Example launch line</title>
- * <para>
- * <programlisting>
- * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! audio/x-raw-int, rate=8000 ! alsasink
- * </programlisting>
- * Decode an Ogg/Vorbis downsample to 8Khz and play sound through alsa. 
- * To create the Ogg/Vorbis file refer to the documentation of vorbisenc.
- * </para>
- * </refsect2>
- *
- * Last reviewed on 2006-03-02 (0.10.4)
- */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -54,18 +35,25 @@ GST_DEBUG_CATEGORY (audioresample_debug);
 #define GST_CAT_DEFAULT audioresample_debug
 
 /* elementfactory information */
-static const GstElementDetails gst_audioresample_details =
+static GstElementDetails gst_audioresample_details =
 GST_ELEMENT_DETAILS ("Audio scaler",
     "Filter/Converter/Audio",
     "Resample audio",
     "David Schleef <ds@schleef.org>");
 
+/* GstAudioresample signals and args */
+enum
+{
+  /* FILL ME */
+  LAST_SIGNAL
+};
+
 #define DEFAULT_FILTERLEN       16
 
 enum
 {
-  PROP_0,
-  PROP_FILTERLEN
+  ARG_0,
+  ARG_FILTERLEN
 };
 
 #define SUPPORTED_CAPS \
@@ -76,56 +64,49 @@ GST_STATIC_CAPS ( \
       "endianness = (int) BYTE_ORDER, " \
       "width = (int) 16, " \
       "depth = (int) 16, " \
-      "signed = (boolean) true;" \
-    "audio/x-raw-int, " \
-      "rate = (int) [ 1, MAX ], " \
-      "channels = (int) [ 1, MAX ], " \
-      "endianness = (int) BYTE_ORDER, " \
-      "width = (int) 32, " \
-      "depth = (int) 32, " \
-      "signed = (boolean) true;" \
-    "audio/x-raw-float, " \
-      "rate = (int) [ 1, MAX ], "	\
-      "channels = (int) [ 1, MAX ], " \
-      "endianness = (int) BYTE_ORDER, " \
-      "width = (int) 32; " \
-    "audio/x-raw-float, " \
-      "rate = (int) [ 1, MAX ], "	\
-      "channels = (int) [ 1, MAX ], " \
-      "endianness = (int) BYTE_ORDER, " \
-      "width = (int) 64" \
+      "signed = (boolean) true " \
 )
 
-static GstStaticPadTemplate gst_audioresample_sink_template =
-GST_STATIC_PAD_TEMPLATE ("sink",
-    GST_PAD_SINK, GST_PAD_ALWAYS, SUPPORTED_CAPS);
+#if 0
+  /* disabled because it segfaults */
+"audio/x-raw-float, "
+    "rate = (int) [ 1, MAX ], "
+    "channels = (int) [ 1, MAX ], "
+    "endianness = (int) BYTE_ORDER, " "width = (int) 32")
+#endif
+     static GstStaticPadTemplate gst_audioresample_sink_template =
+         GST_STATIC_PAD_TEMPLATE ("sink",
+         GST_PAD_SINK, GST_PAD_ALWAYS, SUPPORTED_CAPS);
 
-static GstStaticPadTemplate gst_audioresample_src_template =
-GST_STATIC_PAD_TEMPLATE ("src",
-    GST_PAD_SRC, GST_PAD_ALWAYS, SUPPORTED_CAPS);
+     static GstStaticPadTemplate gst_audioresample_src_template =
+         GST_STATIC_PAD_TEMPLATE ("src",
+         GST_PAD_SRC, GST_PAD_ALWAYS, SUPPORTED_CAPS);
 
-static void gst_audioresample_set_property (GObject * object,
-    guint prop_id, const GValue * value, GParamSpec * pspec);
-static void gst_audioresample_get_property (GObject * object,
-    guint prop_id, GValue * value, GParamSpec * pspec);
+     static void gst_audioresample_dispose (GObject * object);
+
+     static void gst_audioresample_set_property (GObject * object,
+         guint prop_id, const GValue * value, GParamSpec * pspec);
+     static void gst_audioresample_get_property (GObject * object,
+         guint prop_id, GValue * value, GParamSpec * pspec);
 
 /* vmethods */
-gboolean audioresample_get_unit_size (GstBaseTransform * base,
-    GstCaps * caps, guint * size);
-GstCaps *audioresample_transform_caps (GstBaseTransform * base,
-    GstPadDirection direction, GstCaps * caps);
-gboolean audioresample_transform_size (GstBaseTransform * trans,
-    GstPadDirection direction, GstCaps * incaps, guint insize,
-    GstCaps * outcaps, guint * outsize);
-gboolean audioresample_set_caps (GstBaseTransform * base, GstCaps * incaps,
-    GstCaps * outcaps);
-static GstFlowReturn audioresample_pushthrough (GstAudioresample *
-    audioresample);
-static GstFlowReturn audioresample_transform (GstBaseTransform * base,
-    GstBuffer * inbuf, GstBuffer * outbuf);
-static gboolean audioresample_event (GstBaseTransform * base, GstEvent * event);
-static gboolean audioresample_start (GstBaseTransform * base);
-static gboolean audioresample_stop (GstBaseTransform * base);
+     gboolean audioresample_get_unit_size (GstBaseTransform * base,
+         GstCaps * caps, guint * size);
+     GstCaps *audioresample_transform_caps (GstBaseTransform * base,
+         GstPadDirection direction, GstCaps * caps);
+     gboolean audioresample_transform_size (GstBaseTransform * trans,
+         GstPadDirection direction, GstCaps * incaps, guint insize,
+         GstCaps * outcaps, guint * outsize);
+     gboolean audioresample_set_caps (GstBaseTransform * base, GstCaps * incaps,
+         GstCaps * outcaps);
+     static GstFlowReturn audioresample_pushthrough (GstAudioresample *
+         audioresample);
+     static GstFlowReturn audioresample_transform (GstBaseTransform * base,
+         GstBuffer * inbuf, GstBuffer * outbuf);
+     static gboolean audioresample_event (GstBaseTransform * base,
+         GstEvent * event);
+
+/*static guint gst_audioresample_signals[LAST_SIGNAL] = { 0 }; */
 
 #define DEBUG_INIT(bla) \
   GST_DEBUG_CATEGORY_INIT (audioresample_debug, "audioresample", 0, "audio resampling element");
@@ -133,21 +114,20 @@ static gboolean audioresample_stop (GstBaseTransform * base);
 GST_BOILERPLATE_FULL (GstAudioresample, gst_audioresample, GstBaseTransform,
     GST_TYPE_BASE_TRANSFORM, DEBUG_INIT);
 
-static void
-gst_audioresample_base_init (gpointer g_class)
-{
-  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+     static void gst_audioresample_base_init (gpointer g_class)
+     {
+       GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_audioresample_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_audioresample_sink_template));
+       gst_element_class_add_pad_template (gstelement_class,
+           gst_static_pad_template_get (&gst_audioresample_src_template));
+       gst_element_class_add_pad_template (gstelement_class,
+           gst_static_pad_template_get (&gst_audioresample_sink_template));
 
-  gst_element_class_set_details (gstelement_class, &gst_audioresample_details);
-}
+       gst_element_class_set_details (gstelement_class,
+           &gst_audioresample_details);
+     }
 
-static void
-gst_audioresample_class_init (GstAudioresampleClass * klass)
+static void gst_audioresample_class_init (GstAudioresampleClass * klass)
 {
   GObjectClass *gobject_class;
 
@@ -155,16 +135,13 @@ gst_audioresample_class_init (GstAudioresampleClass * klass)
 
   gobject_class->set_property = gst_audioresample_set_property;
   gobject_class->get_property = gst_audioresample_get_property;
+  gobject_class->dispose = gst_audioresample_dispose;
 
-  g_object_class_install_property (gobject_class, PROP_FILTERLEN,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FILTERLEN,
       g_param_spec_int ("filter_length", "filter_length", "filter_length",
           0, G_MAXINT, DEFAULT_FILTERLEN,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
-  GST_BASE_TRANSFORM_CLASS (klass)->start =
-      GST_DEBUG_FUNCPTR (audioresample_start);
-  GST_BASE_TRANSFORM_CLASS (klass)->stop =
-      GST_DEBUG_FUNCPTR (audioresample_stop);
   GST_BASE_TRANSFORM_CLASS (klass)->transform_size =
       GST_DEBUG_FUNCPTR (audioresample_transform_size);
   GST_BASE_TRANSFORM_CLASS (klass)->get_unit_size =
@@ -182,9 +159,10 @@ gst_audioresample_class_init (GstAudioresampleClass * klass)
 }
 
 static void
-gst_audioresample_init (GstAudioresample * audioresample,
+    gst_audioresample_init (GstAudioresample * audioresample,
     GstAudioresampleClass * klass)
 {
+  ResampleState *r;
   GstBaseTransform *trans;
 
   trans = GST_BASE_TRANSFORM (audioresample);
@@ -193,51 +171,37 @@ gst_audioresample_init (GstAudioresample * audioresample,
    * is trivial in the passtrough case. */
   gst_pad_set_bufferalloc_function (trans->sinkpad, NULL);
 
-  audioresample->filter_length = DEFAULT_FILTERLEN;
-}
-
-/* vmethods */
-static gboolean
-audioresample_start (GstBaseTransform * base)
-{
-  GstAudioresample *audioresample = GST_AUDIORESAMPLE (base);
-
-  audioresample->resample = resample_new ();
+  r = resample_new ();
+  audioresample->resample = r;
   audioresample->ts_offset = -1;
   audioresample->offset = -1;
   audioresample->next_ts = -1;
 
-  resample_set_filter_length (audioresample->resample,
-      audioresample->filter_length);
-
-  return TRUE;
+  resample_set_filter_length (r, DEFAULT_FILTERLEN);
+  resample_set_format (r, RESAMPLE_FORMAT_S16);
 }
 
-static gboolean
-audioresample_stop (GstBaseTransform * base)
+static void gst_audioresample_dispose (GObject * object)
 {
-  GstAudioresample *audioresample = GST_AUDIORESAMPLE (base);
+  GstAudioresample *audioresample = GST_AUDIORESAMPLE (object);
 
   if (audioresample->resample) {
     resample_free (audioresample->resample);
     audioresample->resample = NULL;
   }
 
-  gst_caps_replace (&audioresample->sinkcaps, NULL);
-  gst_caps_replace (&audioresample->srccaps, NULL);
-
-  return TRUE;
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+/* vmethods */
 gboolean
-audioresample_get_unit_size (GstBaseTransform * base, GstCaps * caps,
-    guint * size)
-{
+    audioresample_get_unit_size (GstBaseTransform * base, GstCaps * caps,
+    guint * size) {
   gint width, channels;
   GstStructure *structure;
   gboolean ret;
 
-  g_assert (size);
+  g_return_val_if_fail (size, FALSE);
 
   /* this works for both float and int */
   structure = gst_caps_get_structure (caps, 0);
@@ -250,8 +214,7 @@ audioresample_get_unit_size (GstBaseTransform * base, GstCaps * caps,
   return TRUE;
 }
 
-GstCaps *
-audioresample_transform_caps (GstBaseTransform * base,
+GstCaps *audioresample_transform_caps (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps)
 {
   GstCaps *res;
@@ -267,56 +230,35 @@ audioresample_transform_caps (GstBaseTransform * base,
 }
 
 static gboolean
-resample_set_state_from_caps (ResampleState * state, GstCaps * incaps,
+    resample_set_state_from_caps (ResampleState * state, GstCaps * incaps,
     GstCaps * outcaps, gint * channels, gint * inrate, gint * outrate)
 {
   GstStructure *structure;
   gboolean ret;
   gint myinrate, myoutrate;
   int mychannels;
-  gint width, depth;
-  ResampleFormat format;
 
   GST_DEBUG ("incaps %" GST_PTR_FORMAT ", outcaps %"
       GST_PTR_FORMAT, incaps, outcaps);
 
   structure = gst_caps_get_structure (incaps, 0);
 
-  /* get width */
-  ret = gst_structure_get_int (structure, "width", &width);
-  if (!ret)
-    goto no_width;
-
-  /* figure out the format */
+  /* FIXME: once it does float, set the correct format */
+#if 0
   if (g_str_equal (gst_structure_get_name (structure), "audio/x-raw-float")) {
-    if (width == 32)
-      format = RESAMPLE_FORMAT_F32;
-    else if (width == 64)
-      format = RESAMPLE_FORMAT_F64;
-    else
-      goto wrong_depth;
+    r->format = GST_RESAMPLE_FLOAT;
   } else {
-    /* for int, depth and width must be the same */
-    ret = gst_structure_get_int (structure, "depth", &depth);
-    if (!ret || width != depth)
-      goto not_equal;
-
-    if (width == 16)
-      format = RESAMPLE_FORMAT_S16;
-    else if (width == 32)
-      format = RESAMPLE_FORMAT_S32;
-    else
-      goto wrong_depth;
+    r->format = GST_RESAMPLE_S16;
   }
+#endif
+
   ret = gst_structure_get_int (structure, "rate", &myinrate);
   ret &= gst_structure_get_int (structure, "channels", &mychannels);
-  if (!ret)
-    goto no_in_rate_channels;
+  g_return_val_if_fail (ret, FALSE);
 
   structure = gst_caps_get_structure (outcaps, 0);
   ret = gst_structure_get_int (structure, "rate", &myoutrate);
-  if (!ret)
-    goto no_out_rate;
+  g_return_val_if_fail (ret, FALSE);
 
   if (channels)
     *channels = mychannels;
@@ -325,46 +267,17 @@ resample_set_state_from_caps (ResampleState * state, GstCaps * incaps,
   if (outrate)
     *outrate = myoutrate;
 
-  resample_set_format (state, format);
   resample_set_n_channels (state, mychannels);
   resample_set_input_rate (state, myinrate);
   resample_set_output_rate (state, myoutrate);
 
   return TRUE;
-
-  /* ERRORS */
-no_width:
-  {
-    GST_DEBUG ("failed to get width from caps");
-    return FALSE;
-  }
-not_equal:
-  {
-    GST_DEBUG ("width %d and depth %d must be the same", width, depth);
-    return FALSE;
-  }
-wrong_depth:
-  {
-    GST_DEBUG ("unknown depth %d found", depth);
-    return FALSE;
-  }
-no_in_rate_channels:
-  {
-    GST_DEBUG ("could not get input rate and channels");
-    return FALSE;
-  }
-no_out_rate:
-  {
-    GST_DEBUG ("could not get output rate");
-    return FALSE;
-  }
 }
 
 gboolean
-audioresample_transform_size (GstBaseTransform * base,
+    audioresample_transform_size (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps, guint size, GstCaps * othercaps,
-    guint * othersize)
-{
+    guint * othersize) {
   GstAudioresample *audioresample = GST_AUDIORESAMPLE (base);
   ResampleState *state;
   GstCaps *srccaps, *sinkcaps;
@@ -416,9 +329,8 @@ audioresample_transform_size (GstBaseTransform * base,
 }
 
 gboolean
-audioresample_set_caps (GstBaseTransform * base, GstCaps * incaps,
-    GstCaps * outcaps)
-{
+    audioresample_set_caps (GstBaseTransform * base, GstCaps * incaps,
+    GstCaps * outcaps) {
   gboolean ret;
   gint inrate, outrate;
   int channels;
@@ -441,14 +353,16 @@ audioresample_set_caps (GstBaseTransform * base, GstCaps * incaps,
 
   /* save caps so we can short-circuit in the size_transform if the caps
    * are the same */
+  /* FIXME: clean them up in state change ? */
+  gst_caps_ref (incaps);
   gst_caps_replace (&audioresample->sinkcaps, incaps);
+  gst_caps_ref (outcaps);
   gst_caps_replace (&audioresample->srccaps, outcaps);
 
   return TRUE;
 }
 
-static gboolean
-audioresample_event (GstBaseTransform * base, GstEvent * event)
+static gboolean audioresample_event (GstBaseTransform * base, GstEvent * event)
 {
   GstAudioresample *audioresample;
 
@@ -483,7 +397,8 @@ audioresample_event (GstBaseTransform * base, GstEvent * event)
 }
 
 static GstFlowReturn
-audioresample_do_output (GstAudioresample * audioresample, GstBuffer * outbuf)
+    audioresample_do_output (GstAudioresample * audioresample,
+    GstBuffer * outbuf)
 {
   int outsize;
   int outsamples;
@@ -560,7 +475,7 @@ audioresample_do_output (GstAudioresample * audioresample, GstBuffer * outbuf)
 }
 
 static GstFlowReturn
-audioresample_transform (GstBaseTransform * base, GstBuffer * inbuf,
+    audioresample_transform (GstBaseTransform * base, GstBuffer * inbuf,
     GstBuffer * outbuf)
 {
   GstAudioresample *audioresample;
@@ -607,7 +522,7 @@ audioresample_transform (GstBaseTransform * base, GstBuffer * inbuf,
 
 /* push remaining data in the buffers out */
 static GstFlowReturn
-audioresample_pushthrough (GstAudioresample * audioresample)
+    audioresample_pushthrough (GstAudioresample * audioresample)
 {
   int outsize;
   ResampleState *r;
@@ -637,39 +552,38 @@ done:
 
 
 static void
-gst_audioresample_set_property (GObject * object, guint prop_id,
+    gst_audioresample_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstAudioresample *audioresample;
 
-  audioresample = GST_AUDIORESAMPLE (object);
+    g_return_if_fail (GST_IS_AUDIORESAMPLE (object));
+    audioresample = GST_AUDIORESAMPLE (object);
 
   switch (prop_id) {
-    case PROP_FILTERLEN:
+    case ARG_FILTERLEN:
       audioresample->filter_length = g_value_get_int (value);
       GST_DEBUG_OBJECT (GST_ELEMENT (audioresample), "new filter length %d",
           audioresample->filter_length);
-      if (audioresample->resample) {
-        resample_set_filter_length (audioresample->resample,
-            audioresample->filter_length);
-      }
+      resample_set_filter_length (audioresample->resample,
+          audioresample->filter_length);
       break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      default:G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
 static void
-gst_audioresample_get_property (GObject * object, guint prop_id,
+    gst_audioresample_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
   GstAudioresample *audioresample;
 
+  g_return_if_fail (GST_IS_AUDIORESAMPLE (object));
   audioresample = GST_AUDIORESAMPLE (object);
 
   switch (prop_id) {
-    case PROP_FILTERLEN:
+    case ARG_FILTERLEN:
       g_value_set_int (value, audioresample->filter_length);
       break;
     default:
@@ -679,8 +593,7 @@ gst_audioresample_get_property (GObject * object, guint prop_id,
 }
 
 
-static gboolean
-plugin_init (GstPlugin * plugin)
+static gboolean plugin_init (GstPlugin * plugin)
 {
   resample_init ();
 
