@@ -23,9 +23,6 @@
 
 #include <gst/gst.h>
 
-GST_DEBUG_CATEGORY_EXTERN (sub_parse_debug);
-#define GST_CAT_DEFAULT sub_parse_debug
-
 G_BEGIN_DECLS
 
 #define GST_TYPE_SUBPARSE \
@@ -33,10 +30,10 @@ G_BEGIN_DECLS
 #define GST_SUBPARSE(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_SUBPARSE, GstSubParse))
 #define GST_SUBPARSE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_SUBPARSE, GstSubParseClass))
+  (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_SUBPARSE, GstSubParse))
 #define GST_IS_SUBPARSE(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_SUBPARSE))
-#define GST_IS_SUBPARSE_CLASS(klass) \
+#define GST_IS_SUBPARSE_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_SUBPARSE))
 
 typedef struct _GstSubParse GstSubParse;
@@ -48,9 +45,7 @@ typedef enum
   GST_SUB_PARSE_FORMAT_UNKNOWN = 0,
   GST_SUB_PARSE_FORMAT_MDVDSUB = 1,
   GST_SUB_PARSE_FORMAT_SUBRIP = 2,
-  GST_SUB_PARSE_FORMAT_MPSUB = 3,
-  GST_SUB_PARSE_FORMAT_SAMI = 4,
-  GST_SUB_PARSE_FORMAT_TMPLAYER = 5
+  GST_SUB_PARSE_FORMAT_MPSUB = 3
 } GstSubParseFormat;
 
 typedef struct {
@@ -58,9 +53,6 @@ typedef struct {
   GString *buf;
   guint64  start_time;
   guint64  duration;
-  GstSegment *segment;
-  gpointer user_data;
-  gdouble  fps;          /* used by microdvd parser */
 } ParserState;
 
 typedef gchar* (*Parser) (ParserState *state, const gchar *line);
@@ -81,15 +73,6 @@ struct _GstSubParse {
   /* seek */
   guint64 offset;
   guint64 next_offset;
-  
-  /* Segment */
-  GstSegment *segment;
-  GstSeekFlags  segment_flags;
-  gboolean need_segment;
-  
-  gboolean flushing;
-  gboolean valid_utf8;
-  gchar   *encoding;
 };
 
 struct _GstSubParseClass {
