@@ -125,8 +125,6 @@
 #include <gst/interfaces/navigation.h>
 #include <gst/interfaces/xoverlay.h>
 #include <gst/interfaces/colorbalance.h>
-/* Helper functions */
-#include <gst/video/video.h>
 
 /* Object header */
 #include "xvimagesink.h"
@@ -159,7 +157,7 @@ static void gst_xvimagesink_expose (GstXOverlay * overlay);
 //static void gst_xvimagesink_send_pending_navigation (GstXvImageSink * xvimagesink);
 
 /* ElementFactory information */
-static const GstElementDetails gst_xvimagesink_details =
+static GstElementDetails gst_xvimagesink_details =
 GST_ELEMENT_DETAILS ("Video sink",
     "Sink/Video",
     "A Xv based videosink",
@@ -215,8 +213,6 @@ gst_xvimage_buffer_destroy (GstXvImageBuffer * xvimage)
 {
   GstXvImageSink *xvimagesink;
 
-  GST_DEBUG_OBJECT (xvimage, "Destroying buffer");
-
   xvimagesink = xvimage->xvimagesink;
   if (xvimagesink == NULL)
     goto no_sink;
@@ -227,32 +223,17 @@ gst_xvimage_buffer_destroy (GstXvImageBuffer * xvimage)
   if (xvimagesink->cur_image == xvimage)
     xvimagesink->cur_image = NULL;
 
-  /* We might have some buffers destroyed after changing state to NULL */
-  GST_OBJECT_LOCK (xvimagesink);
-  if (xvimagesink->xcontext == NULL) {
-    GST_DEBUG_OBJECT (xvimagesink, "Destroying XvImage after Xcontext");
-#ifdef HAVE_XSHM
-    /* Need to free the shared memory segment even if the x context
-     * was already cleaned up */
-    if (xvimage->SHMInfo.shmaddr != ((void *) -1)) {
-      shmdt (xvimage->SHMInfo.shmaddr);
-    }
-#endif
-    goto beach;
-  }
-
   g_mutex_lock (xvimagesink->x_lock);
 
 #ifdef HAVE_XSHM
   if (xvimagesink->xcontext->use_xshm) {
     if (xvimage->SHMInfo.shmaddr != ((void *) -1)) {
-      GST_DEBUG_OBJECT (xvimagesink, "XServer ShmDetaching from 0x%x id 0x%x\n",
-          xvimage->SHMInfo.shmid, xvimage->SHMInfo.shmseg);
       XShmDetach (xvimagesink->xcontext->disp, &xvimage->SHMInfo);
       XSync (xvimagesink->xcontext->disp, FALSE);
-
       shmdt (xvimage->SHMInfo.shmaddr);
     }
+    if (xvimage->SHMInfo.shmid > 0)
+      shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
     if (xvimage->xvimage)
       XFree (xvimage->xvimage);
   } else
@@ -270,8 +251,6 @@ gst_xvimage_buffer_destroy (GstXvImageBuffer * xvimage)
 
   g_mutex_unlock (xvimagesink->x_lock);
 
-beach:
-  GST_OBJECT_UNLOCK (xvimagesink);
   xvimage->xvimagesink = NULL;
   gst_object_unref (xvimagesink);
 
@@ -288,21 +267,13 @@ static void
 gst_xvimage_buffer_finalize (GstXvImageBuffer * xvimage)
 {
   GstXvImageSink *xvimagesink;
-  gboolean running;
 
   xvimagesink = xvimage->xvimagesink;
-  if (G_UNLIKELY (xvimagesink == NULL))
+  if (xvimagesink == NULL)
     goto no_sink;
 
-  GST_OBJECT_LOCK (xvimagesink);
-  running = xvimagesink->running;
-  GST_OBJECT_UNLOCK (xvimagesink);
-
   /* If our geometry changed we can't reuse that image. */
-  if (running == FALSE) {
-    GST_LOG_OBJECT (xvimage, "destroy image as sink is shutting down");
-    gst_xvimage_buffer_destroy (xvimage);
-  } else if ((xvimage->width != xvimagesink->video_width) ||
+  if ((xvimage->width != xvimagesink->video_width) ||
       (xvimage->height != xvimagesink->video_height)) {
     GST_LOG_OBJECT (xvimage,
         "destroy image as its size changed %dx%d vs current %dx%d",
@@ -405,17 +376,9 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   gint size;
   int (*handler) (Display *, XErrorEvent *);
   gboolean result = FALSE;
-  gboolean did_attach = FALSE;
 
   g_return_val_if_fail (xcontext != NULL, FALSE);
 
-  /* Sync to ensure any older errors are already processed */
-  XSync (xcontext->disp, FALSE);
-
-  /* Set defaults so we don't free these later unnecessarily */
-  SHMInfo.shmaddr = ((void *) -1);
-  SHMInfo.shmid = -1;
-
   /* Setting an error handler to catch failure */
   error_caught = FALSE;
   handler = XSetErrorHandler (gst_xvimagesink_handle_xerror);
@@ -424,10 +387,7 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   GST_DEBUG ("XvShmCreateImage of 1x1");
   xvimage = XvShmCreateImage (xcontext->disp, xcontext->xv_port_id,
       xcontext->im_format, NULL, 1, 1, &SHMInfo);
-
-  /* Might cause an error, sync to ensure it is noticed */
-  XSync (xcontext->disp, FALSE);
-  if (!xvimage || error_caught) {
+  if (!xvimage) {
     GST_WARNING ("could not XvShmCreateImage a 1x1 image");
     goto beach;
   }
@@ -442,16 +402,9 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   SHMInfo.shmaddr = shmat (SHMInfo.shmid, 0, 0);
   if (SHMInfo.shmaddr == ((void *) -1)) {
     GST_WARNING ("Failed to shmat: %s", g_strerror (errno));
-    /* Clean up the shared memory segment */
-    shmctl (SHMInfo.shmid, IPC_RMID, 0);
     goto beach;
   }
 
-  /* Delete the shared memory segment as soon as we manage to attach.
-   * This way, it will be deleted as soon as we detach later, and not
-   * leaked if we crash. */
-  shmctl (SHMInfo.shmid, IPC_RMID, 0);
-
   xvimage->data = SHMInfo.shmaddr;
   SHMInfo.readOnly = FALSE;
 
@@ -460,33 +413,21 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
     goto beach;
   }
 
-  /* Sync to ensure we see any errors we caused */
-  XSync (xcontext->disp, FALSE);
-
-  GST_DEBUG ("XServer ShmAttached to 0x%x, id 0x%x\n", SHMInfo.shmid,
-      SHMInfo.shmseg);
-
-  if (!error_caught) {
-    did_attach = TRUE;
-    /* store whether we succeeded in result */
-    result = TRUE;
-  }
+  /* store whether we succeeded in result and reset error_caught */
+  result = !error_caught;
+  error_caught = FALSE;
 
 beach:
-  /* Sync to ensure we swallow any errors we caused and reset error_caught */
-  XSync (xcontext->disp, FALSE);
-
-  error_caught = FALSE;
   XSetErrorHandler (handler);
 
-  if (did_attach) {
-    GST_DEBUG ("XServer ShmDetaching from 0x%x id 0x%x\n",
-        SHMInfo.shmid, SHMInfo.shmseg);
+  XSync (xcontext->disp, FALSE);
+  if (SHMInfo.shmaddr != ((void *) -1)) {
     XShmDetach (xcontext->disp, &SHMInfo);
     XSync (xcontext->disp, FALSE);
-  }
-  if (SHMInfo.shmaddr != ((void *) -1))
     shmdt (SHMInfo.shmaddr);
+  }
+  if (SHMInfo.shmid > 0)
+    shmctl (SHMInfo.shmid, IPC_RMID, 0);
   if (xvimage)
     XFree (xvimage);
   return result;
@@ -503,8 +444,8 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink, GstCaps * caps)
 
   g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), NULL);
 
+
   xvimage = (GstXvImageBuffer *) gst_mini_object_new (GST_TYPE_XVIMAGE_BUFFER);
-  GST_DEBUG_OBJECT (xvimage, "Creating new XvImageBuffer");
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -517,12 +458,9 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink, GstCaps * caps)
       xvimage->height);
 
   xvimage->im_format = gst_xvimagesink_get_format_from_caps (xvimagesink, caps);
-  if (xvimage->im_format == -1) {
+  if (!xvimage->im_format) {
     GST_WARNING_OBJECT (xvimagesink, "failed to get format from caps %"
         GST_PTR_FORMAT, caps);
-    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE,
-        ("Failed to create output image buffer of %dx%d pixels",
-            xvimage->width, xvimage->height), ("Invalid input caps"));
     goto beach_unlocked;
   }
   xvimage->xvimagesink = gst_object_ref (xvimagesink);
@@ -536,13 +474,9 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink, GstCaps * caps)
         xvimage->im_format, NULL,
         xvimage->width, xvimage->height, &xvimage->SHMInfo);
     if (!xvimage->xvimage) {
-      g_mutex_unlock (xvimagesink->x_lock);
-      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE,
-          ("Failed to create output image buffer of %dx%d pixels",
-              xvimage->width, xvimage->height),
-          ("could not XvShmCreateImage a %dx%d image",
-              xvimage->width, xvimage->height));
-      goto beach_unlocked;
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
+          ("could not XvShmCreateImage a %dx%d image"));
+      goto beach;
     }
 
     /* we have to use the returned data_size for our shm size */
@@ -552,45 +486,28 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink, GstCaps * caps)
     xvimage->SHMInfo.shmid = shmget (IPC_PRIVATE, xvimage->size,
         IPC_CREAT | 0777);
     if (xvimage->SHMInfo.shmid == -1) {
-      g_mutex_unlock (xvimagesink->x_lock);
-      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE,
-          ("Failed to create output image buffer of %dx%d pixels",
-              xvimage->width, xvimage->height),
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
           ("could not get shared memory of %d bytes", xvimage->size));
-      goto beach_unlocked;
+      goto beach;
     }
 
     xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, 0, 0);
     if (xvimage->SHMInfo.shmaddr == ((void *) -1)) {
-      g_mutex_unlock (xvimagesink->x_lock);
-      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE,
-          ("Failed to create output image buffer of %dx%d pixels",
-              xvimage->width, xvimage->height),
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
           ("Failed to shmat: %s", g_strerror (errno)));
-      /* Clean up the shared memory segment */
-      shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
-      goto beach_unlocked;
+      goto beach;
     }
 
-    /* Delete the shared memory segment as soon as we manage to attach.
-     * This way, it will be deleted as soon as we detach later, and not
-     * leaked if we crash. */
-    shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
-
     xvimage->xvimage->data = xvimage->SHMInfo.shmaddr;
     xvimage->SHMInfo.readOnly = FALSE;
 
     if (XShmAttach (xvimagesink->xcontext->disp, &xvimage->SHMInfo) == 0) {
-      g_mutex_unlock (xvimagesink->x_lock);
-      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE,
-          ("Failed to create output image buffer of %dx%d pixels",
-              xvimage->width, xvimage->height), ("Failed to XShmAttach"));
-      goto beach_unlocked;
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
+          ("Failed to XShmAttach"));
+      goto beach;
     }
 
     XSync (xvimagesink->xcontext->disp, FALSE);
-    GST_DEBUG_OBJECT (xvimagesink, "XServer ShmAttached to 0x%x, id 0x%x\n",
-        xvimage->SHMInfo.shmid, xvimage->SHMInfo.shmseg);
   } else
 #endif /* HAVE_XSHM */
   {
@@ -598,13 +515,9 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink, GstCaps * caps)
         xvimagesink->xcontext->xv_port_id,
         xvimage->im_format, NULL, xvimage->width, xvimage->height);
     if (!xvimage->xvimage) {
-      g_mutex_unlock (xvimagesink->x_lock);
-      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE,
-          ("Failed to create outputimage buffer of %dx%d pixels",
-              xvimage->width, xvimage->height),
-          ("could not XvCreateImage a %dx%d image",
-              xvimage->width, xvimage->height));
-      goto beach_unlocked;
+      GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
+          ("could not XvCreateImage a %dx%d image"));
+      goto beach;
     }
 
     /* we have to use the returned data_size for our image size */
@@ -618,6 +531,7 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink, GstCaps * caps)
   GST_BUFFER_DATA (xvimage) = (guchar *) xvimage->xvimage->data;
   GST_BUFFER_SIZE (xvimage) = xvimage->size;
 
+beach:
   g_mutex_unlock (xvimagesink->x_lock);
 
 beach_unlocked:
@@ -686,8 +600,7 @@ gst_xvimagesink_xvimage_put (GstXvImageSink * xvimagesink,
       gst_buffer_unref (xvimagesink->cur_image);
     }
     GST_LOG_OBJECT (xvimagesink, "reffing %p as our current image", xvimage);
-    xvimagesink->cur_image =
-        GST_XVIMAGE_BUFFER (gst_buffer_ref (GST_BUFFER (xvimage)));
+    xvimagesink->cur_image = GST_XVIMAGE_BUFFER (gst_buffer_ref (xvimage));
   }
 
   /* Expose sends a NULL image, we take the latest frame */
@@ -726,11 +639,9 @@ gst_xvimagesink_xvimage_put (GstXvImageSink * xvimagesink,
 #ifdef HAVE_XSHM
   if (xvimagesink->xcontext->use_xshm) {
     GST_LOG_OBJECT (xvimagesink,
-        "XvShmPutImage with image %dx%d and window %dx%d, from xvimage %"
-        GST_PTR_FORMAT,
+        "XvShmPutImage with image %dx%d and window %dx%d",
         xvimage->width, xvimage->height,
-        xvimagesink->xwindow->width, xvimagesink->xwindow->height, xvimage);
-
+        xvimagesink->xwindow->width, xvimagesink->xwindow->height);
     XvShmPutImage (xvimagesink->xcontext->disp,
         xvimagesink->xcontext->xv_port_id,
         xvimagesink->xwindow->win,
@@ -1106,18 +1017,13 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
   XvAdaptorInfo *adaptors;
   gint nb_formats;
   XvImageFormatValues *formats = NULL;
-  guint nb_encodings;
-  XvEncodingInfo *encodings = NULL;
-  gulong max_w = G_MAXINT, max_h = G_MAXINT;
   GstCaps *caps = NULL;
-  GstCaps *rgb_caps = NULL;
 
   g_return_val_if_fail (xcontext != NULL, NULL);
 
   /* First let's check that XVideo extension is available */
   if (!XQueryExtension (xcontext->disp, "XVideo", &i, &i, &i)) {
-    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, SETTINGS,
-        ("Could not initialise Xv output"),
+    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, SETTINGS, (NULL),
         ("XVideo extension is not available"));
     return NULL;
   }
@@ -1125,8 +1031,7 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
   /* Then we get adaptors list */
   if (Success != XvQueryAdaptors (xcontext->disp, xcontext->root,
           &nb_adaptors, &adaptors)) {
-    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, SETTINGS,
-        ("Could not initialise Xv output"),
+    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, SETTINGS, (NULL),
         ("Failed getting XV adaptors list"));
     return NULL;
   }
@@ -1156,19 +1061,17 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
   XvFreeAdaptorInfo (adaptors);
 
   if (!xcontext->xv_port_id) {
-    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, BUSY,
-        ("Could not initialise Xv output"), ("No port available"));
+    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, BUSY, (NULL),
+        ("No port available"));
     return NULL;
   }
 
-  /* Set XV_AUTOPAINT_COLORKEY and XV_DOUBLE_BUFFER and XV_COLORKEY */
+  /* Set XV_AUTOPAINT_COLORKEY */
   {
     int count;
     XvAttribute *const attr = XvQueryPortAttributes (xcontext->disp,
         xcontext->xv_port_id, &count);
     static const char autopaint[] = "XV_AUTOPAINT_COLORKEY";
-    static const char dbl_buffer[] = "XV_DOUBLE_BUFFER";
-    static const char colorkey[] = "XV_COLORKEY";
 
     for (i = 0; i < count; i++)
       if (!strcmp (attr[i].name, autopaint)) {
@@ -1178,54 +1081,15 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
         break;
       }
 
-    for (i = 0; i < count; i++)
-      if (!strcmp (attr[i].name, dbl_buffer)) {
-        const Atom atom = XInternAtom (xcontext->disp, dbl_buffer, False);
-
-        XvSetPortAttribute (xcontext->disp, xcontext->xv_port_id, atom, 1);
-        break;
-      }
-
-    /* Set the colorkey to something that is dark but hopefully won't randomly
-     * appear on the screen elsewhere (ie not black or greys) */
-    for (i = 0; i < count; i++)
-      if (!strcmp (attr[i].name, colorkey)) {
-        const Atom atom = XInternAtom (xcontext->disp, colorkey, False);
-
-        XvSetPortAttribute (xcontext->disp, xcontext->xv_port_id, atom,
-            0x010203);
-        break;
-      }
-
     XFree (attr);
   }
 
-  /* Get the list of encodings supported by the adapter and look for the
-   * XV_IMAGE encoding so we can determine the maximum width and height
-   * supported */
-  XvQueryEncodings (xcontext->disp, xcontext->xv_port_id, &nb_encodings,
-      &encodings);
-
-  for (i = 0; i < nb_encodings; i++) {
-    GST_LOG_OBJECT (xvimagesink,
-        "Encoding %d, name %s, max wxh %lux%lu rate %d/%d",
-        i, encodings[i].name, encodings[i].width, encodings[i].height,
-        encodings[i].rate.numerator, encodings[i].rate.denominator);
-    if (strcmp (encodings[i].name, "XV_IMAGE") == 0) {
-      max_w = encodings[i].width;
-      max_h = encodings[i].height;
-    }
-  }
-
-  XvFreeEncodingInfo (encodings);
-
   /* We get all image formats supported by our port */
   formats = XvListImageFormats (xcontext->disp,
       xcontext->xv_port_id, &nb_formats);
   caps = gst_caps_new_empty ();
   for (i = 0; i < nb_formats; i++) {
     GstCaps *format_caps = NULL;
-    gboolean is_rgb_format = FALSE;
 
     /* We set the image format of the xcontext to an existing one. Sink
        connect method will override that but we need to have at least a
@@ -1236,44 +1100,23 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
     switch (formats[i].type) {
       case XvRGB:
       {
-        XvImageFormatValues *fmt = &(formats[i]);
-        gint endianness = G_BIG_ENDIAN;
-
-        if (fmt->byte_order == LSBFirst) {
-          /* our caps system handles 24/32bpp RGB as big-endian. */
-          if (fmt->bits_per_pixel == 24 || fmt->bits_per_pixel == 32) {
-            fmt->red_mask = GUINT32_TO_BE (fmt->red_mask);
-            fmt->green_mask = GUINT32_TO_BE (fmt->green_mask);
-            fmt->blue_mask = GUINT32_TO_BE (fmt->blue_mask);
-
-            if (fmt->bits_per_pixel == 24) {
-              fmt->red_mask >>= 8;
-              fmt->green_mask >>= 8;
-              fmt->blue_mask >>= 8;
-            }
-          } else
-            endianness = G_LITTLE_ENDIAN;
-        }
-
         format_caps = gst_caps_new_simple ("video/x-raw-rgb",
-            "endianness", G_TYPE_INT, endianness,
-            "depth", G_TYPE_INT, fmt->depth,
-            "bpp", G_TYPE_INT, fmt->bits_per_pixel,
-            "blue_mask", G_TYPE_INT, fmt->red_mask,
-            "green_mask", G_TYPE_INT, fmt->green_mask,
-            "red_mask", G_TYPE_INT, fmt->blue_mask,
-            "width", GST_TYPE_INT_RANGE, 1, max_w,
-            "height", GST_TYPE_INT_RANGE, 1, max_h,
+            "endianness", G_TYPE_INT, xcontext->endianness,
+            "depth", G_TYPE_INT, xcontext->depth,
+            "bpp", G_TYPE_INT, xcontext->bpp,
+            "blue_mask", G_TYPE_INT, formats[i].red_mask,
+            "green_mask", G_TYPE_INT, formats[i].green_mask,
+            "red_mask", G_TYPE_INT, formats[i].blue_mask,
+            "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+            "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
             "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT, 1, NULL);
-
-        is_rgb_format = TRUE;
         break;
       }
       case XvYUV:
         format_caps = gst_caps_new_simple ("video/x-raw-yuv",
             "format", GST_TYPE_FOURCC, formats[i].id,
-            "width", GST_TYPE_INT_RANGE, 1, max_w,
-            "height", GST_TYPE_INT_RANGE, 1, max_h,
+            "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+            "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
             "framerate", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT, 1, NULL);
         break;
       default:
@@ -1290,21 +1133,10 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
         format->caps = gst_caps_copy (format_caps);
         xcontext->formats_list = g_list_append (xcontext->formats_list, format);
       }
-
-      if (is_rgb_format) {
-        if (rgb_caps == NULL)
-          rgb_caps = format_caps;
-        else
-          gst_caps_append (rgb_caps, format_caps);
-      } else
-        gst_caps_append (caps, format_caps);
     }
-  }
 
-  /* Collected all caps into either the caps or rgb_caps structures.
-   * Append rgb_caps on the end of YUV, so that YUV is always preferred */
-  if (rgb_caps)
-    gst_caps_append (caps, rgb_caps);
+    gst_caps_append (caps, format_caps);
+  }
 
   if (formats)
     XFree (formats);
@@ -1346,10 +1178,7 @@ gst_xvimagesink_calculate_pixel_aspect_ratio (GstXContext * xcontext)
     {1, 1},                     /* regular screen */
     {16, 15},                   /* PAL TV */
     {11, 10},                   /* 525 line Rec.601 video */
-    {54, 59},                   /* 625 line Rec.601 video */
-    {64, 45},                   /* 1280x1024 on 16:9 display */
-    {5, 3},                     /* 1280x1024 on 4:3 display */
-    {4, 3}                      /*  800x600 on 16:9 display */
+    {54, 59}                    /* 625 line Rec.601 video */
   };
   gint i;
   gint index;
@@ -1421,8 +1250,8 @@ gst_xvimagesink_xcontext_get (GstXvImageSink * xvimagesink)
   if (!xcontext->disp) {
     g_mutex_unlock (xvimagesink->x_lock);
     g_free (xcontext);
-    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE,
-        ("Could not initialise Xv output"), ("Could not open display"));
+    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
+        ("Could not open display"));
     return NULL;
   }
 
@@ -1451,8 +1280,8 @@ gst_xvimagesink_xcontext_get (GstXvImageSink * xvimagesink)
     XCloseDisplay (xcontext->disp);
     g_mutex_unlock (xvimagesink->x_lock);
     g_free (xcontext);
-    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, SETTINGS,
-        ("Could not initialise Xv output"), ("Could not get pixel formats"));
+    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, SETTINGS, (NULL),
+        ("Could not get pixel formats"));
     return NULL;
   }
 
@@ -1570,21 +1399,9 @@ static void
 gst_xvimagesink_xcontext_clear (GstXvImageSink * xvimagesink)
 {
   GList *formats_list, *channels_list;
-  GstXContext *xcontext;
 
   g_return_if_fail (GST_IS_XVIMAGESINK (xvimagesink));
-
-  GST_OBJECT_LOCK (xvimagesink);
-  if (xvimagesink->xcontext == NULL) {
-    GST_OBJECT_UNLOCK (xvimagesink);
-    return;
-  }
-
-  /* Take the XContext from the sink and clean it up */
-  xcontext = xvimagesink->xcontext;
-  xvimagesink->xcontext = NULL;
-
-  GST_OBJECT_UNLOCK (xvimagesink);
+  g_return_if_fail (xvimagesink->xcontext != NULL);
 
   /* Wait for our event thread */
   if (xvimagesink->event_thread) {
@@ -1592,7 +1409,7 @@ gst_xvimagesink_xcontext_clear (GstXvImageSink * xvimagesink)
     xvimagesink->event_thread = NULL;
   }
 
-  formats_list = xcontext->formats_list;
+  formats_list = xvimagesink->xcontext->formats_list;
 
   while (formats_list) {
     GstXvImageFormat *format = formats_list->data;
@@ -1602,10 +1419,10 @@ gst_xvimagesink_xcontext_clear (GstXvImageSink * xvimagesink)
     formats_list = g_list_next (formats_list);
   }
 
-  if (xcontext->formats_list)
-    g_list_free (xcontext->formats_list);
+  if (xvimagesink->xcontext->formats_list)
+    g_list_free (xvimagesink->xcontext->formats_list);
 
-  channels_list = xcontext->channels_list;
+  channels_list = xvimagesink->xcontext->channels_list;
 
   while (channels_list) {
     GstColorBalanceChannel *channel = channels_list->data;
@@ -1614,26 +1431,23 @@ gst_xvimagesink_xcontext_clear (GstXvImageSink * xvimagesink)
     channels_list = g_list_next (channels_list);
   }
 
-  if (xcontext->channels_list)
-    g_list_free (xcontext->channels_list);
+  if (xvimagesink->xcontext->channels_list)
+    g_list_free (xvimagesink->xcontext->channels_list);
 
-  gst_caps_unref (xcontext->caps);
-  if (xcontext->last_caps)
-    gst_caps_replace (&xcontext->last_caps, NULL);
-
-  g_free (xcontext->par);
+  gst_caps_unref (xvimagesink->xcontext->caps);
+  g_free (xvimagesink->xcontext->par);
 
   g_mutex_lock (xvimagesink->x_lock);
 
-  GST_DEBUG_OBJECT (xvimagesink, "Closing display and freeing X Context");
+  XvUngrabPort (xvimagesink->xcontext->disp,
+      xvimagesink->xcontext->xv_port_id, 0);
 
-  XvUngrabPort (xcontext->disp, xcontext->xv_port_id, 0);
-
-  XCloseDisplay (xcontext->disp);
+  XCloseDisplay (xvimagesink->xcontext->disp);
 
   g_mutex_unlock (xvimagesink->x_lock);
 
-  g_free (xcontext);
+  g_free (xvimagesink->xcontext);
+  xvimagesink->xcontext = NULL;
 }
 
 static void
@@ -1677,12 +1491,11 @@ gst_xvimagesink_get_format_from_caps (GstXvImageSink * xvimagesink,
         gst_caps_unref (icaps);
         return format->format;
       }
-      gst_caps_unref (icaps);
     }
     list = g_list_next (list);
   }
 
-  return -1;
+  return 0;
 }
 
 static GstCaps *
@@ -1711,9 +1524,10 @@ gst_xvimagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   gint video_width, video_height;
   gint video_par_n, video_par_d;        /* video's PAR */
   gint display_par_n, display_par_d;    /* display's PAR */
+  GValue display_ratio = { 0, };        /* display w/h ratio */
   const GValue *caps_par;
   const GValue *fps;
-  guint num, den;
+  gint num, den;
 
   xvimagesink = GST_XVIMAGESINK (bsink);
 
@@ -1736,11 +1550,8 @@ gst_xvimagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   fps = gst_structure_get_value (structure, "framerate");
   ret &= (fps != NULL);
 
-  if (!ret) {
-    GST_DEBUG_OBJECT (xvimagesink, "Failed to retrieve either width, "
-        "height or framerate from intersected caps");
+  if (!ret)
     return FALSE;
-  }
 
   xvimagesink->fps_n = gst_value_get_fraction_numerator (fps);
   xvimagesink->fps_d = gst_value_get_fraction_denominator (fps);
@@ -1748,15 +1559,15 @@ gst_xvimagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   xvimagesink->video_width = video_width;
   xvimagesink->video_height = video_height;
   im_format = gst_xvimagesink_get_format_from_caps (xvimagesink, caps);
-  if (im_format == -1) {
-    GST_DEBUG_OBJECT (xvimagesink,
-        "Could not locate image format from caps %" GST_PTR_FORMAT, caps);
+  if (im_format == 0) {
     return FALSE;
   }
 
   /* get aspect ratio from caps if it's present, and
    * convert video width and height to a display width and height
-   * using wd / hd = wv / hv * PARv / PARd */
+   * using wd / hd = wv / hv * PARv / PARd
+   * the ratio wd / hd will be stored in display_ratio */
+  g_value_init (&display_ratio, GST_TYPE_FRACTION);
 
   /* get video's PAR */
   caps_par = gst_structure_get_value (structure, "pixel-aspect-ratio");
@@ -1776,14 +1587,12 @@ gst_xvimagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
     display_par_d = 1;
   }
 
-  if (!gst_video_calculate_display_ratio (&num, &den, video_width,
-          video_height, video_par_n, video_par_d, display_par_n,
-          display_par_d)) {
-    GST_ELEMENT_ERROR (xvimagesink, CORE, NEGOTIATION, (NULL),
-        ("Error calculating the output display ratio of the video."));
-    return FALSE;
-  }
+  gst_value_set_fraction (&display_ratio,
+      video_width * video_par_n * display_par_d,
+      video_height * video_par_d * display_par_n);
 
+  num = gst_value_get_fraction_numerator (&display_ratio);
+  den = gst_value_get_fraction_denominator (&display_ratio);
   GST_DEBUG_OBJECT (xvimagesink,
       "video width/height: %dx%d, calculated display ratio: %d/%d",
       video_width, video_height, num, den);
@@ -1796,18 +1605,15 @@ gst_xvimagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   /* check hd / den is an integer scale factor, and scale wd with the PAR */
   if (video_height % den == 0) {
     GST_DEBUG_OBJECT (xvimagesink, "keeping video height");
-    GST_VIDEO_SINK_WIDTH (xvimagesink) = (guint)
-        gst_util_uint64_scale_int (video_height, num, den);
+    GST_VIDEO_SINK_WIDTH (xvimagesink) = video_height * num / den;
     GST_VIDEO_SINK_HEIGHT (xvimagesink) = video_height;
   } else if (video_width % num == 0) {
     GST_DEBUG_OBJECT (xvimagesink, "keeping video width");
     GST_VIDEO_SINK_WIDTH (xvimagesink) = video_width;
-    GST_VIDEO_SINK_HEIGHT (xvimagesink) = (guint)
-        gst_util_uint64_scale_int (video_width, den, num);
+    GST_VIDEO_SINK_HEIGHT (xvimagesink) = video_width * den / num;
   } else {
     GST_DEBUG_OBJECT (xvimagesink, "approximating while keeping video height");
-    GST_VIDEO_SINK_WIDTH (xvimagesink) = (guint)
-        gst_util_uint64_scale_int (video_height, num, den);
+    GST_VIDEO_SINK_WIDTH (xvimagesink) = video_height * num / den;
     GST_VIDEO_SINK_HEIGHT (xvimagesink) = video_height;
   }
   GST_DEBUG_OBJECT (xvimagesink, "scaling to %dx%d",
@@ -1819,13 +1625,8 @@ gst_xvimagesink_setcaps (GstBaseSink * bsink, GstCaps * caps)
   }
 
   /* Creating our window and our image with the display size in pixels */
-  if (GST_VIDEO_SINK_WIDTH (xvimagesink) <= 0 ||
-      GST_VIDEO_SINK_HEIGHT (xvimagesink) <= 0) {
-    GST_ELEMENT_ERROR (xvimagesink, CORE, NEGOTIATION, (NULL),
-        ("Error calculating the output display ratio of the video."));
-    return FALSE;
-  }
-
+  g_assert (GST_VIDEO_SINK_WIDTH (xvimagesink) > 0);
+  g_assert (GST_VIDEO_SINK_HEIGHT (xvimagesink) > 0);
   if (!xvimagesink->xwindow)
     xvimagesink->xwindow = gst_xvimagesink_xwindow_new (xvimagesink,
         GST_VIDEO_SINK_WIDTH (xvimagesink),
@@ -1856,25 +1657,16 @@ gst_xvimagesink_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
   GstXvImageSink *xvimagesink;
-  GstXContext *xcontext = NULL;
 
   xvimagesink = GST_XVIMAGESINK (element);
 
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
-      /* Initializing the XContext */
-      if (xvimagesink->xcontext == NULL) {
-        xcontext = gst_xvimagesink_xcontext_get (xvimagesink);
-        if (xcontext == NULL)
-          return GST_STATE_CHANGE_FAILURE;
-      }
-
-      GST_OBJECT_LOCK (xvimagesink);
       xvimagesink->running = TRUE;
-      if (xcontext)
-        xvimagesink->xcontext = xcontext;
-      GST_OBJECT_UNLOCK (xvimagesink);
-
+      /* Initializing the XContext */
+      if (!xvimagesink->xcontext &&
+          !(xvimagesink->xcontext = gst_xvimagesink_xcontext_get (xvimagesink)))
+        return GST_STATE_CHANGE_FAILURE;
       /* update object's par with calculated one if not set yet */
       if (!xvimagesink->par) {
         xvimagesink->par = g_new0 (GValue, 1);
@@ -1911,9 +1703,7 @@ gst_xvimagesink_change_state (GstElement * element, GstStateChange transition)
       GST_VIDEO_SINK_HEIGHT (xvimagesink) = 0;
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
-      GST_OBJECT_LOCK (xvimagesink);
       xvimagesink->running = FALSE;
-      GST_OBJECT_UNLOCK (xvimagesink);
       if (xvimagesink->cur_image) {
         gst_buffer_unref (xvimagesink->cur_image);
         xvimagesink->cur_image = NULL;
@@ -1922,15 +1712,18 @@ gst_xvimagesink_change_state (GstElement * element, GstStateChange transition)
         gst_buffer_unref (xvimagesink->xvimage);
         xvimagesink->xvimage = NULL;
       }
-
-      gst_xvimagesink_imagepool_clear (xvimagesink);
+      if (xvimagesink->image_pool)
+        gst_xvimagesink_imagepool_clear (xvimagesink);
 
       if (xvimagesink->xwindow) {
         gst_xvimagesink_xwindow_destroy (xvimagesink, xvimagesink->xwindow);
         xvimagesink->xwindow = NULL;
       }
 
-      gst_xvimagesink_xcontext_clear (xvimagesink);
+      if (xvimagesink->xcontext) {
+        gst_xvimagesink_xcontext_clear (xvimagesink);
+        xvimagesink->xcontext = NULL;
+      }
       break;
     default:
       break;
@@ -1982,19 +1775,7 @@ gst_xvimagesink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
           GST_BUFFER_CAPS (buf));
 
       if (!xvimagesink->xvimage)
-        /* The create method should have posted an informative error */
         goto no_image;
-
-      if (xvimagesink->xvimage->size < GST_BUFFER_SIZE (buf)) {
-        GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE,
-            ("Failed to create output image buffer of %dx%d pixels",
-                xvimagesink->xvimage->width, xvimagesink->xvimage->height),
-            ("XServer allocated buffer size did not match input buffer"));
-
-        gst_xvimage_buffer_destroy (xvimagesink->xvimage);
-        xvimagesink->xvimage = NULL;
-        goto no_image;
-      }
     }
 
     memcpy (xvimagesink->xvimage->xvimage->data,
@@ -2011,6 +1792,8 @@ no_image:
   {
     /* No image available. That's very bad ! */
     GST_WARNING_OBJECT (xvimagesink, "could not create image");
+    GST_ELEMENT_ERROR (xvimagesink, CORE, NEGOTIATION, (NULL),
+        ("Failed creating an XvImage in xvimagesink chain function."));
     return GST_FLOW_ERROR;
   }
 }
@@ -2021,112 +1804,11 @@ static GstFlowReturn
 gst_xvimagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
     GstCaps * caps, GstBuffer ** buf)
 {
-  GstFlowReturn ret = GST_FLOW_OK;
   GstXvImageSink *xvimagesink;
   GstXvImageBuffer *xvimage = NULL;
-  GstCaps *intersection = NULL;
-  GstStructure *structure = NULL;
-  gint width, height, image_format;
 
   xvimagesink = GST_XVIMAGESINK (bsink);
 
-  if (G_LIKELY (xvimagesink->xcontext->last_caps &&
-          gst_caps_is_equal (caps, xvimagesink->xcontext->last_caps))) {
-    GST_DEBUG_OBJECT (xvimagesink,
-        "buffer alloc for same last_caps, reusing caps");
-    intersection = gst_caps_ref (caps);
-    image_format = xvimagesink->xcontext->last_format;
-
-    goto reuse_last_caps;
-  }
-
-  GST_DEBUG_OBJECT (xvimagesink, "buffer alloc requested with caps %"
-      GST_PTR_FORMAT "intersecting with our caps %" GST_PTR_FORMAT, caps,
-      xvimagesink->xcontext->caps);
-
-  /* Check the caps against our xcontext */
-  intersection = gst_caps_intersect (xvimagesink->xcontext->caps, caps);
-
-  /* Ensure the returned caps are fixed */
-  gst_caps_truncate (intersection);
-
-  GST_DEBUG_OBJECT (xvimagesink, "intersection in buffer alloc returned %"
-      GST_PTR_FORMAT, intersection);
-
-  if (gst_caps_is_empty (intersection)) {
-    /* So we don't support this kind of buffer, let's define one we'd like */
-    GstCaps *new_caps = gst_caps_copy (caps);
-
-    structure = gst_caps_get_structure (new_caps, 0);
-
-    /* Try with YUV first */
-    gst_structure_set_name (structure, "video/x-raw-yuv");
-    gst_structure_remove_field (structure, "format");
-    gst_structure_remove_field (structure, "endianness");
-    gst_structure_remove_field (structure, "depth");
-    gst_structure_remove_field (structure, "bpp");
-    gst_structure_remove_field (structure, "red_mask");
-    gst_structure_remove_field (structure, "green_mask");
-    gst_structure_remove_field (structure, "blue_mask");
-    gst_structure_remove_field (structure, "alpha_mask");
-
-    /* Reuse intersection with Xcontext */
-    gst_caps_unref (intersection);
-    intersection = gst_caps_intersect (xvimagesink->xcontext->caps, new_caps);
-
-    if (gst_caps_is_empty (intersection)) {
-      /* Now try with RGB */
-      gst_structure_set_name (structure, "video/x-raw-rgb");
-      /* And interset again */
-      gst_caps_unref (intersection);
-      intersection = gst_caps_intersect (xvimagesink->xcontext->caps, new_caps);
-
-      if (gst_caps_is_empty (intersection)) {
-        GST_WARNING_OBJECT (xvimagesink, "we were requested a buffer with "
-            "caps %" GST_PTR_FORMAT "but our xcontext caps %" GST_PTR_FORMAT
-            "are completely incompatible with those caps", new_caps,
-            xvimagesink->xcontext->caps);
-        gst_caps_unref (new_caps);
-        ret = GST_FLOW_UNEXPECTED;
-        goto beach;
-      }
-    }
-
-    /* Clean this copy */
-    gst_caps_unref (new_caps);
-    /* We want fixed caps */
-    gst_caps_truncate (intersection);
-
-    GST_DEBUG_OBJECT (xvimagesink, "allocating a buffer with caps %"
-        GST_PTR_FORMAT, intersection);
-  } else if (gst_caps_is_equal (intersection, caps)) {
-    /* Things work better if we return a buffer with the same caps ptr
-     * as was asked for when we can */
-    gst_caps_replace (&intersection, caps);
-  }
-
-  /* Get image format from caps */
-  image_format = gst_xvimagesink_get_format_from_caps (xvimagesink,
-      intersection);
-
-  /* Store our caps and format as the last_caps to avoid expensive 
-   * caps intersection next time */
-  gst_caps_replace (&xvimagesink->xcontext->last_caps, intersection);
-  xvimagesink->xcontext->last_format = image_format;
-
-reuse_last_caps:
-
-  /* Get geometry from caps */
-  structure = gst_caps_get_structure (intersection, 0);
-  if (!gst_structure_get_int (structure, "width", &width) ||
-      !gst_structure_get_int (structure, "height", &height) ||
-      image_format == -1) {
-    GST_WARNING_OBJECT (xvimagesink, "invalid caps for buffer allocation %"
-        GST_PTR_FORMAT, intersection);
-    ret = GST_FLOW_UNEXPECTED;
-    goto beach;
-  }
-
   g_mutex_lock (xvimagesink->pool_lock);
 
   /* Walking through the pool cleaning unusable images and searching for a
@@ -2140,8 +1822,9 @@ reuse_last_caps:
           xvimagesink->image_pool);
 
       /* We check for geometry or image format changes */
-      if ((xvimage->width != width) ||
-          (xvimage->height != height) || (xvimage->im_format != image_format)) {
+      if ((xvimage->width != xvimagesink->video_width) ||
+          (xvimage->height != xvimagesink->video_height) ||
+          (xvimage->im_format != xvimagesink->xcontext->im_format)) {
         /* This image is unusable. Destroying... */
         gst_xvimage_buffer_free (xvimage);
         xvimage = NULL;
@@ -2158,26 +1841,14 @@ reuse_last_caps:
   if (!xvimage) {
     /* We found no suitable image in the pool. Creating... */
     GST_DEBUG_OBJECT (xvimagesink, "no usable image in pool, creating xvimage");
-    xvimage = gst_xvimagesink_xvimage_new (xvimagesink, intersection);
-    if (xvimage && xvimage->size < size) {
-      /* This image is unusable. Destroying... */
-      gst_xvimage_buffer_free (xvimage);
-      xvimage = NULL;
-    }
+    xvimage = gst_xvimagesink_xvimage_new (xvimagesink, caps);
   }
-
   if (xvimage) {
-    gst_buffer_set_caps (GST_BUFFER (xvimage), intersection);
+    gst_buffer_set_caps (GST_BUFFER (xvimage), caps);
   }
-
   *buf = GST_BUFFER (xvimage);
 
-beach:
-  if (intersection) {
-    gst_caps_unref (intersection);
-  }
-
-  return ret;
+  return GST_FLOW_OK;
 }
 
 /* Interfaces stuff */
@@ -2534,7 +2205,7 @@ gst_xvimagesink_get_property (GObject * object, guint prop_id,
       g_value_set_int (value, xvimagesink->saturation);
       break;
     case ARG_DISPLAY:
-      g_value_set_string (value, xvimagesink->display_name);
+      g_value_set_string (value, g_strdup (xvimagesink->display_name));
       break;
     case ARG_SYNCHRONOUS:
       g_value_set_boolean (value, xvimagesink->synchronous);
@@ -2639,7 +2310,7 @@ gst_xvimagesink_class_init (GstXvImageSinkClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
+  parent_class = g_type_class_ref (GST_TYPE_VIDEO_SINK);
 
   gobject_class->set_property = gst_xvimagesink_set_property;
   gobject_class->get_property = gst_xvimagesink_get_property;
@@ -2747,10 +2418,6 @@ gst_xvimagesink_get_type (void)
         &overlay_info);
     g_type_add_interface_static (xvimagesink_type, GST_TYPE_COLOR_BALANCE,
         &colorbalance_info);
-
-    /* make sure type is registered and not at runtime as this is not
-     * threadsafe */
-    gst_xvimage_buffer_get_type ();
   }
 
   return xvimagesink_type;
